@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2018-2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Affine constraint types."""
@@ -26,13 +23,13 @@ from collections import namedtuple
 from .. import glyphs
 from ..apidoc import api_end, api_start
 from ..caching import cached_property
-from .constraint import Constraint, ConstraintConversion
+from .constraint import ConicConstraint, ConstraintConversion
 
 _API_START = api_start(globals())
 # -------------------------------
 
 
-class AffineConstraint(Constraint):
+class AffineConstraint(ConicConstraint):
     """An equality or inequality between two affine expressions."""
 
     def __init__(self, lhs, relation, rhs, customString=None):
@@ -61,6 +58,19 @@ class AffineConstraint(Constraint):
         super(AffineConstraint, self).__init__(
             "Affine", customString, printSize=True)
 
+    @cached_property
+    def conic_membership_form(self):
+        """Implement for :class:`~.constraint.ConicConstraint`."""
+        from ..expressions import NonnegativeOrthant, ZeroSpace
+
+        element = self.ge0.vec
+        dim = len(element)
+
+        if self.relation == self.EQ:
+            return element, ZeroSpace(dim=dim)
+        else:
+            return element, NonnegativeOrthant(dim=dim)
+
     Subtype = namedtuple("Subtype", ("dim", "eq"))
 
     def _subtype(self):
@@ -129,9 +139,9 @@ class AffineConstraint(Constraint):
 
     def _get_slack(self):
         if self.relation == self.LE:
-            delta = self.rhs.value - self.lhs.value
+            delta = self.rhs.safe_value - self.lhs.safe_value
         else:
-            delta = self.lhs.value - self.rhs.value
+            delta = self.lhs.safe_value - self.rhs.safe_value
 
         return -abs(delta) if self.relation == self.EQ else delta
 
@@ -181,7 +191,7 @@ class AffineConstraint(Constraint):
         return rows
 
 
-class ComplexAffineConstraint(Constraint):
+class ComplexAffineConstraint(ConicConstraint):
     """An equality between affine expressions, at least one being complex."""
 
     class RealConversion(ConstraintConversion):
@@ -238,6 +248,12 @@ class ComplexAffineConstraint(Constraint):
         super(ComplexAffineConstraint, self).__init__(
             "Complex Equality", customString, printSize=True)
 
+    @cached_property
+    def conic_membership_form(self):
+        """Implement for :class:`~.constraint.ConicConstraint`."""
+        from ..expressions import ZeroSpace
+        return self.lhs - self.rhs, ZeroSpace()
+
     Subtype = namedtuple("Subtype", ("dim",))
 
     def _subtype(self):
@@ -258,7 +274,7 @@ class ComplexAffineConstraint(Constraint):
         return self.lhs.size
 
     def _get_slack(self):
-        return -abs(self.lhs.value - self.rhs.value)
+        return -abs(self.lhs.safe_value - self.rhs.safe_value)
 
 
 # --------------------------------------
