@@ -271,11 +271,11 @@ class NuclearNormConstraint(Constraint):
             t = con.upperBound
 
             if con.norm._complex:
-                Y = HermitianVariable("Y", (m, m))
-                Z = HermitianVariable("Z", (n, n))
+                Y = HermitianVariable("__Y", (m, m))
+                Z = HermitianVariable("__Z", (n, n))
             else:
-                Y = SymmetricVariable("Y", (m, m))
-                Z = SymmetricVariable("Z", (n, n))
+                Y = SymmetricVariable("__Y", (m, m))
+                Z = SymmetricVariable("__Z", (n, n))
 
             P = Problem()
             if x.hermitian:
