@@ -127,6 +127,7 @@ class PowerTraceConstraint(Constraint):
                 yield ("var", RealVariable.make_var_type(dim=1, bnd=0), N - 1)
                 yield ("con", RSOCConstraint.make_type(argdim=1), N)
                 if hasM:
+                    yield ("var", RealVariable.make_var_type(dim=1, bnd=0), 1)
                     yield ("con",
                         AffineConstraint.make_type(dim=1, eq=False), 1)
             else:
@@ -161,8 +162,12 @@ class PowerTraceConstraint(Constraint):
 
             if n == 1:
                 idt = Constant('1', 1)
-                varcnt = 0
-                v = []
+                if m is None:
+                    varcnt = 0
+                    v = []
+                else:
+                    varcnt = 1
+                    v = [P.add_variable('__v[0]', 1)]
             else:
                 idt = Constant('I', cvx.spdiag([1.] * n))
                 varcnt = 1
