@@ -22,6 +22,7 @@
 import builtins
 import functools
 import itertools
+from collections.abc import Iterator
 
 import cvxopt
 import numpy
@@ -102,47 +103,69 @@ def J(rows=1, cols=1):
     return AffineExpression.from_constant(1, (rows, cols))
 
 
-def sum(lst, it=None, indices=None):
+def sum(lst, axis=None):
     """Sum PICOS expressions and give the result a meaningful description.
 
     This is a replacement for Python's :func:`sum` that produces sensible string
-    representations when summing PICOS expressions.
+    representations when summing over multiple PICOS expressions. Additionally,
+    this can be used to denote the (complete, column- or row-) sum over a single
+    matrix expression.
 
-    :param lst: A list of
-        :class:`~picos.expressions.ComplexAffineExpression`, or a
-        single affine expression whose elements shall be summed.
-    :type lst: list or tuple or
-        ~picos.expressions.ComplexAffineExpression
-
-    :param it: DEPRECATED
-    :param indices: DEPRECATED
+    :param None or int axis:
+        If summing over a single matrix expression, this is the axis over which
+        the sum is performed: :obj:`None` denotes the sum over all elements,
+        ``0`` denotes the sum of the rows as a row vector and ``1`` denotes the
+        sum of the columns as a column vector. If summing multiple expressions,
+        any value other than :obj:`None` raises a :exc:`ValueError`.
 
     :Example:
 
-    >>> import builtins
-    >>> import picos
+    >>> import builtins, picos, numpy
     >>> x = picos.RealVariable("x", 5)
     >>> e = [x[i]*x[i+1] for i in range(len(x) - 1)]
     >>> builtins.sum(e)
     <Quadratic Expression: x[0]·x[1] + x[1]·x[2] + x[2]·x[3] + x[3]·x[4]>
     >>> picos.sum(e)
     <Quadratic Expression: ∑(x[i]·x[i+1] : i ∈ [0…3])>
-    >>> picos.sum(x)  # The same as (x|1).
+    >>> picos.sum(x)  # The same as x.sum or (x|1).
     <1×1 Real Linear Expression: ∑(x)>
+    >>> A = picos.Constant("A", range(20), (4, 5))
+    >>> picos.sum(A, axis=0)  # The same as A.rowsum
+    <1×5 Real Constant: [1]ᵀ·A>
+    >>> picos.sum(A, axis=1)  # The same as A.colsum
+    <4×1 Real Constant: A·[1]>
+    >>> numpy.allclose(  # Same axis convention as NumPy.
+    ...     numpy.sum(A.np, axis=0), picos.sum(A, axis=0).np)
+    True
     """
-    if it is not None or indices is not None:
-        # Deprecated as of 2.0.
-        throw_deprecation_warning("Arguments 'it' and 'indices' to picos.sum "
-            "are deprecated and ignored.")
-
     if isinstance(lst, Expression):
         if isinstance(lst, BiaffineExpression):
-            return (lst | 1.0)
+            if axis is None:
+                return lst.sum
+            elif not isinstance(axis, (int, numpy.integer)):
+                raise TypeError(
+                    "Axis must be an integer or None. (To sum multiple "
+                    "expressions, provide an iterable as the first argument.)")
+            elif axis == 0:
+                return lst.rowsum
+            elif axis == 1:
+                return lst.colsum
+            else:
+                raise ValueError("Bad axis: {}.".format(axis))
         else:
             raise TypeError(
                 "PICOS doesn't know how to sum over a single {}."
                 .format(type(lst).__name__))
 
+    if axis is not None:
+        raise ValueError(
+            "No axis may be given when summing multiple expressions.")
+
+    # Allow passing also an iterator instead of an iterable. (The conversion is
+    # necessary as otherwise the check below would use up the iterator.)
+    if isinstance(lst, Iterator):
+        lst = tuple(lst)
+
     # Resort to Python's built-in sum when no summand is a PICOS expression.
     if not any(isinstance(expression, Expression) for expression in lst):
         return builtins.sum(lst)
@@ -155,7 +178,7 @@ def sum(lst, it=None, indices=None):
             "PICOS constant.") from error
 
     if len(lst) == 0:
-        return Constant(0)
+        return O()
     elif len(lst) == 1:
         return lst[0]
     elif len(lst) == 2:
