@@ -125,6 +125,12 @@ _LOAD_DATA_REGEX = re.compile("^{}{}{}$".format(
 ))
 
 
+def is_scipy_spmat(value):
+    """Report whether value is a SciPy sparse matrix without importing scipy."""
+    return hasattr(value, "__module__") \
+        and value.__module__.startswith("scipy.sparse")
+
+
 def load_data(value, shape=None, typecode=None, sparse=None,
         alwaysCopy=True, legacy=False):
     r"""Load a constant numeric data value as a CVXOPT (sparse) matrix.
@@ -138,6 +144,7 @@ def load_data(value, shape=None, typecode=None, sparse=None,
     :Supported data types:
 
         - A NumPy matrix: :class:`numpy.ndarray` or :class:`numpy.matrix`.
+        - A SciPy sparse matrix: All from :mod:`scipy.sparse`.
         - A CVXOPT matrix: :obj:`cvxopt.matrix` or :obj:`cvxopt.spmatrix`.
         - A constant PICOS expression: :class:`~.exp_affine.AffineExpression` or
           :class:`~.exp_affine.ComplexAffineExpression`.
@@ -408,6 +415,11 @@ def load_data(value, shape=None, typecode=None, sparse=None,
     if isinstance(value, Expression):
         value = value.refined
 
+    # Convert from a SciPy sparse to a CVXOPT sparse matrix.
+    if is_scipy_spmat(value):
+        value = sp2cvx(value)
+        alwaysCopy = False
+
     # Convert the data to a CVXOPT (sparse) matrix of proper shape and type.
     if isinstance(value, (int, float, complex, numpy.number)):
         # Convert NumPy numbers to Python ones.
@@ -813,7 +825,8 @@ def load_data(value, shape=None, typecode=None, sparse=None,
             string = glyphs.matrix(
                 glyphs.compsep(glyphs.shape(outShape), string))
         elif base.startswith("|"):
-            string = glyphs.matrix(string)
+            if outShape != (1, 1):
+                string = glyphs.matrix(string)
         elif base == "I":
             pass
         else:
@@ -984,8 +997,9 @@ def convert_and_refine_arguments(*which, refine=True, allowNone=False):
 
 
 def convert_operands(
-        sameShape=False, scalarLHS=False, scalarRHS=False, rMatMul=False,
-        lMatMul=False, horiCat=False, vertCat=False, allowNone=False):
+        sameShape=False, diagBroadcast=False, scalarLHS=False, scalarRHS=False,
+        rMatMul=False, lMatMul=False, horiCat=False, vertCat=False,
+        allowNone=False):
     """Convert binary operator operands to PICOS expressions.
 
     A decorator for a binary operator that converts any operand that is not
@@ -1017,9 +1031,13 @@ def convert_operands(
     Currently, only scalar affine expressions are broadcasted to the next
     smallest matching shape. This is more limited than the broadcasting behavior
     when one of the operands is raw data, but it ensures a symmetric behavior in
-    case both operands are affine.
+    case both operands are affine. An exception is the case of diagBroadcast,
+    where a vector affine expression may be extended to a matrix.
 
     :param bool sameShape: Both operands must have the exact same shape.
+    :param bool diagBroadcast: Both operands must be square matrices of same
+        shape. If one operand is a square matrix and the other is a scalar or
+        vector, the latter is put into the diagonal of a square matrix.
     :param bool scalarLHS: The left hand side operand must be scalar.
     :param bool scalarRHS: The right hand side operand must be scalar.
     :param bool rMatMul: The operation has the shape requirements of normal
@@ -1045,10 +1063,11 @@ def convert_operands(
         scalarLHS = True
         scalarRHS = True
 
-    # Check arguments, only scalarLHS and scalarRHS may appear together.
+    # Check arguments; only scalarLHS and scalarRHS may appear together.
     anyScalar = scalarLHS or scalarRHS
-    selected  = len([arg for arg in
-        (sameShape, anyScalar, lMatMul, rMatMul, horiCat, vertCat) if arg])
+    args = (
+        sameShape, diagBroadcast, anyScalar, lMatMul, rMatMul, horiCat, vertCat)
+    selected  = len([arg for arg in args if arg])
     if selected > 1:
         assert False, "Conflicting convert_operands arguments."
 
@@ -1064,7 +1083,7 @@ def convert_operands(
                     opName, lhsName, rhsName, reason)) from error
 
             def make_lhs_shape(rhsShape):
-                if sameShape:
+                if sameShape or diagBroadcast:
                     return rhsShape
                 elif horiCat:
                     return (rhsShape[0], None)
@@ -1080,7 +1099,7 @@ def convert_operands(
                     return None
 
             def make_rhs_shape(lhsShape):
-                if sameShape:
+                if sameShape or diagBroadcast:
                     return lhsShape
                 elif horiCat:
                     return (lhsShape[0], None)
@@ -1138,9 +1157,12 @@ def convert_operands(
 
                 lm, ln = lhs.shape
                 rm, rn = rhs.shape
+                lhsSquare = lm == ln
+                rhsSquare = rm == rn
 
                 # Further check if already matching size.
-                if (horiCat and lm == rm) \
+                if (diagBroadcast and lhsShape == rhsShape and lhsSquare) \
+                or (horiCat and lm == rm) \
                 or (vertCat and ln == rn) \
                 or (rMatMul and ln == rm) \
                 or (lMatMul and rn == lm):
@@ -1164,12 +1186,32 @@ def convert_operands(
 
                 # Broadcast an affine scalar on the left side to match.
                 if lhsIsAffine and lhsL == 1:
-                    lhs = lhs.broadcasted(make_lhs_shape(rhsShape))
+                    if diagBroadcast and rhsSquare:
+                        lhs = lhs.dupdiag(rm)
+                    else:
+                        lhs = lhs.broadcasted(make_lhs_shape(rhsShape))
                     return operator(lhs, rhs, *args, **kwargs)
 
                 # Broadcast an affine scalar on the right side to match.
                 if rhsIsAffine and rhsL == 1:
-                    rhs = rhs.broadcasted(make_rhs_shape(lhsShape))
+                    if diagBroadcast and lhsSquare:
+                        rhs = rhs.dupdiag(lm)
+                    else:
+                        rhs = rhs.broadcasted(make_rhs_shape(lhsShape))
+                    return operator(lhs, rhs, *args, **kwargs)
+
+                # Diagonally broadcast an affine vector on the left hand side.
+                if diagBroadcast and lhsIsAffine and rhsSquare \
+                and 1 in lhsShape and rm in lhsShape:
+                    lhs = lhs.diag
+                    assert lhs.shape == rhs.shape
+                    return operator(lhs, rhs, *args, **kwargs)
+
+                # Diagonally broadcast an affine vector on the right hand side.
+                if diagBroadcast and rhsIsAffine and lhsSquare \
+                and 1 in rhsShape and lm in rhsShape:
+                    rhs = rhs.diag
+                    assert lhs.shape == rhs.shape
                     return operator(lhs, rhs, *args, **kwargs)
 
                 # At least one of the expressions is affine and we didn't find a
@@ -1182,8 +1224,25 @@ def convert_operands(
 
                 rhsShape = None if lhsIsSet else make_rhs_shape(lhs.shape)
 
+                if diagBroadcast and lhs.shape[0] != lhs.shape[1]:
+                    fail("Given that the right hand side operand is not a PICOS"
+                        " expression, the left hand side operand must be square"
+                        " for this operation.")
+
                 try:
-                    if rMatMul or lMatMul:
+                    if diagBroadcast:
+                        try:
+                            rhs = Constant(rhs, shape=(1, 1))
+                        except Exception:
+                            try:
+                                rhs = Constant(rhs, shape=(lhs.shape[0], 1))
+                            except Exception:
+                                rhs = Constant(rhs, shape=rhsShape)
+                            else:
+                                rhs = rhs.diag
+                        else:
+                            rhs = rhs.dupdiag(lhs.shape[0])
+                    elif rMatMul or lMatMul:
                         if lhs.shape == (1, 1):
                             # Any shape works.
                             rhs = Constant(rhs)
@@ -1203,7 +1262,24 @@ def convert_operands(
 
                 lhsShape = None if rhsIsSet else make_lhs_shape(rhs.shape)
 
+                if diagBroadcast and rhs.shape[0] != rhs.shape[1]:
+                    fail("Given that the left hand side operand is not a PICOS "
+                        "expression, the right hand side operand must be square"
+                        " for this operation.")
+
                 try:
+                    if diagBroadcast:
+                        try:
+                            lhs = Constant(lhs, shape=(1, 1))
+                        except Exception:
+                            try:
+                                lhs = Constant(lhs, shape=(rhs.shape[0], 1))
+                            except Exception:
+                                lhs = Constant(lhs, shape=lhsShape)
+                            else:
+                                lhs = lhs.diag
+                        else:
+                            lhs = lhs.dupdiag(rhs.shape[0])
                     if rMatMul or lMatMul:
                         if rhs.shape == (1, 1):
                             # Any shape works.
@@ -1391,7 +1467,7 @@ def cvxopt_principal_root(matrix):
         Does not validate that the input matrix is symmetric positive
         semidefinite and will still return a (useless) matrix if it is not.
     """
-    matrix_np = numpy.array(cvx2np(matrix))  # TODO: GitLab issue #239.
+    matrix_np = cvx2np(matrix)
     U, s, _ = numpy.linalg.svd(matrix_np, hermitian=True)
     root_np = numpy.dot(U*(s**0.5), U.T)
     root, _ = load_data(root_np, matrix.size)
@@ -1551,7 +1627,7 @@ def right_kronecker_I(A, k, reshape=None, preT=False, postT=False):
 
 
 def cvx2np(A, reshape=None):
-    """Convert a CVXOPT (sparse) matrix to a NumPy matrix.
+    """Convert a CVXOPT (sparse) matrix to a NumPy two-dimensional array.
 
     :param A: The CVXOPT :func:`dense <cvxopt:cvxopt.matrix>` or
         :func:`sparse <cvxopt:cvxopt.spmatrix>` matrix to convert.
@@ -1568,7 +1644,27 @@ def cvx2np(A, reshape=None):
         shape = load_shape(reshape)
         return numpy.reshape(A, shape, "F")
     else:
-        return numpy.matrix(A)
+        return numpy.array(A)
+
+
+def sp2cvx(A):
+    """Convert a SciPy sparse matrix to a CVXOPT sparse matrix."""
+    import scipy.sparse
+
+    assert isinstance(A, scipy.sparse.spmatrix)
+
+    A = A.tocoo()
+
+    V = A.data
+    I = A.row
+    J = A.col
+
+    if issubclass(A.dtype.type, numpy.complex):
+        tc = "z"
+    else:
+        tc = "d"
+
+    return cvxopt.spmatrix(V, I, J, A.shape, tc)
 
 
 def make_fraction(p, denominator_limit):
