@@ -19,6 +19,8 @@
 
 """Implements :class:`Entropy` and :class:`NegativeEntropy`."""
 
+# TODO: Common base class for Entropy and NegativeEntropy.
+
 import math
 import operator
 from collections import namedtuple
@@ -192,7 +194,12 @@ class Entropy(Expression):
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __add__(self, other):
+        other_str = other.string
+
         if isinstance(other, AffineExpression):
+            if other.is0:
+                return self
+
             other = Entropy(other, other * math.e)
 
         if isinstance(other, Entropy):
@@ -210,33 +217,32 @@ class Entropy(Expression):
                 entropy = Entropy(
                     self._x.vec // other._x.vec, self._y.vec // other._y.vec)
 
-            entropy._symbStr = glyphs.clever_add(self.string, other.string)
+            entropy._symbStr = glyphs.clever_add(self.string, other_str)
 
             return entropy
-        else:
-            return NotImplemented
 
-    @convert_operands(scalarRHS=True)
-    @refine_operands()
-    def __sub__(self, other):
-        if isinstance(other, AffineExpression):
-            other = NegativeEntropy(other, other / math.e)
-
-        if isinstance(other, NegativeEntropy):
-            return self + (-other)
-        else:
-            return NotImplemented
+        return Expression.__add__(self, other)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __radd__(self, other):
         if isinstance(other, (AffineExpression, Entropy)):
-            entropy = self.__add__(other)
-            # NOTE: __add__ always creates a fresh expression.
-            entropy._symbStr = glyphs.clever_add(other.string, self.string)
+            entropy = Entropy.__add__(self, other)
+
+            if entropy is not self:
+                entropy._symbStr = glyphs.clever_add(other.string, self.string)
+
             return entropy
-        else:
-            return NotImplemented
+
+        return Expression.__radd__(self, other)
+
+    @convert_operands(scalarRHS=True)
+    @refine_operands()
+    def __sub__(self, other):
+        if isinstance(other, (AffineExpression, NegativeEntropy)):
+            return self + (-other)
+
+        return Expression.__sub__(self, other)
 
     # --------------------------------------------------------------------------
     # Methods and properties that return expressions.
@@ -432,7 +438,12 @@ class NegativeEntropy(Expression):
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __add__(self, other):
+        other_str = other.string
+
         if isinstance(other, AffineExpression):
+            if other.is0:
+                return self
+
             other = NegativeEntropy(other, other / math.e)
 
         if isinstance(other, NegativeEntropy):
@@ -450,33 +461,32 @@ class NegativeEntropy(Expression):
                 negent = NegativeEntropy(
                     self._x.vec // other._x.vec, self._y.vec // other._y.vec)
 
-            negent._symbStr = glyphs.clever_add(self.string, other.string)
+            negent._symbStr = glyphs.clever_add(self.string, other_str)
 
             return negent
-        else:
-            return NotImplemented
+
+        return Expression.__add__(self, other)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __radd__(self, other):
         if isinstance(other, (AffineExpression, NegativeEntropy)):
-            negent = self.__add__(other)
-            # NOTE: __add__ always creates a fresh expression.
-            negent._symbStr = glyphs.clever_add(other.string, self.string)
+            negent = NegativeEntropy.__add__(self, other)
+
+            if negent is not self:
+                negent._symbStr = glyphs.clever_add(other.string, self.string)
+
             return negent
-        else:
-            return NotImplemented
+
+        return Expression.__radd__(self, other)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __sub__(self, other):
-        if isinstance(other, AffineExpression):
-            other = Entropy(other, other * math.e)
-
-        if isinstance(other, Entropy):
+        if isinstance(other, (AffineExpression, Entropy)):
             return self + (-other)
-        else:
-            return NotImplemented
+
+        return Expression.__sub__(self, other)
 
     # --------------------------------------------------------------------------
     # Methods and properties that return expressions.
