@@ -110,75 +110,77 @@ class Logarithm(Expression):
     # Python special method implementations, except constraint-creating ones.
     # --------------------------------------------------------------------------
 
+    @classmethod
+    def _add(cls, self, other, forward):
+        if isinstance(other, AffineExpression) and other.constant:
+            if other.is0:
+                return self
+
+            log = cls(self._x * math.exp(other.value))
+            log._typeStr = "Offset " + log._typeStr
+
+            if forward:
+                log._symbStr = glyphs.clever_add(self.string, other.string)
+            else:
+                log._symbStr = glyphs.clever_add(other.string, self.string)
+
+            return log
+
+        if forward:
+            return Expression.__add__(self, other)
+        else:
+            return Expression.__radd__(self, other)
+
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __add__(self, other):
-        if isinstance(other, AffineExpression):
-            if not other.constant:
-                raise NotImplementedError("You may only add a constant term to "
-                    "a nonconstant PICOS logarithm.")
-
-            log = Logarithm(self._x * math.exp(other.value))
-            log._typeStr = "Offset " + log._typeStr
-            log._symbStr = glyphs.clever_add(self.string, other.string)
-
-            return log
+        return Logarithm._add(self, other, True)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __radd__(self, other):
-        if isinstance(other, AffineExpression):
-            log = self.__add__(other)
-            # NOTE: __add__ always creates a fresh expression.
-            log._symbStr = glyphs.clever_add(other.string, self.string)
-            return log
-        else:
-            return NotImplemented
+        return Logarithm._add(self, other, False)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __sub__(self, other):
-        if isinstance(other, AffineExpression):
-            if not other.constant:
-                raise NotImplementedError("You may only substract a constant "
-                    "term from a nonconstant PICOS logarithm.")
-
+        if isinstance(other, AffineExpression) and other.constant:
             log = Logarithm(self._x / math.exp(other.value))
             log._typeStr = "Offset " + log._typeStr
             log._symbStr = glyphs.clever_sub(self.string, other.string)
 
             return log
 
-    @convert_operands(scalarRHS=True)
-    @refine_operands()
-    def __mul__(self, other):
+        return Expression.__sub__(self, other)
+
+    @classmethod
+    def _mul(cls, self, other, forward):
         from . import Entropy, NegativeEntropy
 
         if isinstance(other, AffineExpression):
             if other.is0:
                 return AffineExpression.zero()
             elif other.is1:
-                # NOTE: We could return self here, but this is more consistent
-                #       with other expressions' __mul__ methods.
-                return Logarithm(self._x)
+                return self
             elif other.equals(self._x):
                 return NegativeEntropy(self._x)
             elif other.equals(-self._x):
                 return Entropy(self._x)
-            else:
-                raise NotImplementedError(
-                    "You may multiply {} only with {}, 0, or 1."
-                    .format(self.string, glyphs.plsmns(self._x.string)))
+
+        if forward:
+            return Expression.__mul__(self, other)
         else:
-            return NotImplemented
+            return Expression.__rmul__(self, other)
+
+    @convert_operands(scalarRHS=True)
+    @refine_operands()
+    def __mul__(self, other):
+        return Logarithm._mul(self, other, True)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __rmul__(self, other):
-        if isinstance(other, AffineExpression):
-            return self.__mul__(other)
-        else:
-            return NotImplemented
+        return Logarithm._mul(self, other, False)
 
     # --------------------------------------------------------------------------
     # Methods and properties that return expressions.
