@@ -114,36 +114,49 @@ class LogSumExp(Expression):
     # Python special method implementations, except constraint-creating ones.
     # --------------------------------------------------------------------------
 
+    @classmethod
+    def _add(cls, self, other, forward):
+        if isinstance(other, AffineExpression):
+            if other.is0:
+                return self
+
+            lse = cls(self._x + other)
+
+            if forward:
+                lse._symbStr = glyphs.clever_add(self.string, other.string)
+            else:
+                lse._symbStr = glyphs.clever_add(other.string, self.string)
+
+            return lse
+
+        if forward:
+            return Expression.__add__(self, other)
+        else:
+            return Expression.__radd__(self, other)
+
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __add__(self, other):
-        if isinstance(other, AffineExpression):
-            log = LogSumExp(self._x + other)
-            log._symbStr = glyphs.clever_add(self.string, other.string)
-            return log
-        else:
-            return NotImplemented
+        return LogSumExp._add(self, other, True)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __radd__(self, other):
-        if isinstance(other, AffineExpression):
-            log = self.__add__(other)
-            # NOTE: __add__ always creates a fresh expression.
-            log._symbStr = glyphs.clever_add(other.string, self.string)
-            return log
-        else:
-            return NotImplemented
+        return LogSumExp._add(self, other, False)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __sub__(self, other):
         if isinstance(other, AffineExpression):
-            log = LogSumExp(self._x - other)
-            log._symbStr = glyphs.clever_sub(self.string, other.string)
-            return log
-        else:
-            return NotImplemented
+            if other.is0:
+                return self
+
+            lse = LogSumExp(self._x - other)
+            lse._symbStr = glyphs.clever_sub(self.string, other.string)
+
+            return lse
+
+        return Expression.__sub__(self, other)
 
     # --------------------------------------------------------------------------
     # Methods and properties that return expressions.
