@@ -135,7 +135,7 @@ class PowerTrace(Expression):
             if not m.constant:
                 raise TypeError("The additional factor {} is not constant."
                     .format(m.string))
-            elif not cvxopt_hpsd(m.value):
+            elif not cvxopt_hpsd(m.safe_value_as_matrix):
                 raise ValueError("The additional factor {} is not hermitian "
                     "positive semidefinite.".format(m.string))
 
@@ -242,39 +242,37 @@ class PowerTrace(Expression):
     # Python special method implementations, except constraint-creating ones.
     # --------------------------------------------------------------------------
 
+    @classmethod
+    def _mul(cls, self, other, forward):
+        if isinstance(other, AffineExpression) and other.constant:
+            factor = other.safe_value
+
+            if not factor:
+                return AffineExpression.zero()
+            elif factor == 1:
+                return self
+            elif factor > 0 and self.p >= 0 and self.p <= 1:
+                if self._m is None:
+                    m = other.dupdiag(self.n).renamed(other.string)
+                else:
+                    m = other*self._m
+
+                return cls(self._x, self.p, m, self._limit)
+
+        if forward:
+            return Expression.__mul__(self, other)
+        else:
+            return Expression.__rmul__(self, other)
+
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __mul__(self, other):
-        if isinstance(other, AffineExpression):
-            if not other.constant:
-                raise NotImplementedError("You may only multiply a nonconstant "
-                    "PICOS trace of a power with a constant term.")
-
-            value = other.value
-
-            if value < 0:
-                raise NotImplementedError("You may only multiply a nonconstant "
-                    "PICOS trace of a power with a nonnegative term.")
-
-            if value == 1:
-                return self
-
-            if self._m is None:
-                m = other.dupdiag(self.n).renamed(other.string)
-            else:
-                m = other*self._m
-
-            return PowerTrace(self._x, self.p, m, self._limit)
-        else:
-            return NotImplemented
+        return PowerTrace._mul(self, other, True)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __rmul__(self, other):
-        if isinstance(other, AffineExpression):
-            return self.__mul__(other)
-        else:
-            return NotImplemented
+        return PowerTrace._mul(self, other, False)
 
     # --------------------------------------------------------------------------
     # Methods and properties that return expressions.
