@@ -117,7 +117,7 @@ class QuadraticExpression(Expression):
         assert not scalarFactors or (
             isinstance(scalarFactors, (tuple, list)) and len(scalarFactors) == 2
             and all(isinstance(x, AffineExpression) for x in scalarFactors)
-            and all(len(x) == 1 for x in scalarFactors))
+            and all(x.scalar for x in scalarFactors))
 
         Expression.__init__(self, "Quadratic Expression", string)
 
@@ -288,152 +288,176 @@ class QuadraticExpression(Expression):
     # Python special method implementations, except constraint-creating ones.
     # --------------------------------------------------------------------------
 
-    def __len__(self):
-        # Faster version that overrides Expression.__len__.
-        return 1
+    @classmethod
+    def _add_sub(cls, self, other, add, forward):
+        def affine_part_and_string(other_aff):
+            if add:
+                if forward:
+                    affine = self._aff + other_aff
+                    string = glyphs.clever_add(self.string, other.string)
+                else:
+                    affine = other_aff + self._aff
+                    string = glyphs.clever_add(other.string, self.string)
+            else:
+                if forward:
+                    affine = self._aff - other_aff
+                    string = glyphs.clever_sub(self.string, other.string)
+                else:
+                    affine = other_aff - self._aff
+                    string = glyphs.clever_sub(other.string, self.string)
+
+            return affine, string
+
+        if isinstance(other, cls):
+            affine, string = affine_part_and_string(other._aff)
+
+            quads = {}
+            varPairs = set(self._quads.keys()).union(other._quads.keys())
+            for vars in varPairs:
+                if vars in self._quads:
+                    if vars in other._quads:
+                        if add:
+                            quads[vars] = self._quads[vars] + other._quads[vars]
+                        elif forward:
+                            quads[vars] = self._quads[vars] - other._quads[vars]
+                        else:
+                            quads[vars] = other._quads[vars] - self._quads[vars]
+                    elif not add and not forward:
+                        quads[vars] = -self._quads[vars]
+                    else:
+                        quads[vars] = self._quads[vars]
+                elif not add and forward:
+                    quads[vars] = -other._quads[vars]
+                else:
+                    quads[vars] = other._quads[vars]
+
+            return cls(string, quads, affine)
+        elif isinstance(other, AffineExpression):
+            affine, string = affine_part_and_string(other)
+
+            if add or forward:
+                quads = self._quads
+                copy_ = self
+            else:
+                quads = {xy: -A for xy, A in self._quads.items()}
+                copy_ = None
+
+            return cls(string, quads, affine, copyDecomposition=copy_)
+
+        if add:
+            if forward:
+                return Expression.__add__(self, other)
+            else:
+                return Expression.__radd__(self, other)
+        else:
+            if forward:
+                return Expression.__sub__(self, other)
+            else:
+                return Expression.__rsub__(self, other)
 
     @convert_operands(sameShape=True)
     def __add__(self, other):
         """Denote addition from the right hand side."""
-        if isinstance(other, QuadraticExpression):
-            string = glyphs.clever_add(self.string, other.string)
-
-            quadPart = {}
-            varPairs = set(self._quads.keys()).union(other._quads.keys())
-            for vars in varPairs:
-                if vars in self._quads:
-                    if vars in other._quads:
-                        quadPart[vars] = self._quads[vars] + other._quads[vars]
-                    else:
-                        quadPart[vars] = self._quads[vars]
-                else:
-                    quadPart[vars] = other._quads[vars]
-
-            affPart = self._aff + other._aff
-
-            return QuadraticExpression(string, quadPart, affPart)
-        elif isinstance(other, AffineExpression):
-            string = glyphs.clever_add(self.string, other.string)
-
-            return QuadraticExpression(
-                string, self._quads, self._aff + other, copyDecomposition=self)
-        else:
-            return NotImplemented
+        return QuadraticExpression._add_sub(self, other, True, True)
 
     @convert_operands(sameShape=True)
     def __radd__(self, other):
         """Denote addition from the left hand side."""
-        if isinstance(other, QuadraticExpression):
-            assert False, "Impossible case."
-        elif isinstance(other, AffineExpression):
-            result = self.__add__(other)
-            # NOTE: __add__ always creates a fresh expression.
-            result._symbStr = glyphs.clever_add(other.string, self.string)
-            return result
-        else:
-            return NotImplemented
+        return QuadraticExpression._add_sub(self, other, True, False)
 
     @convert_operands(sameShape=True)
     def __sub__(self, other):
         """Denote substraction from the right hand side."""
-        if isinstance(other, QuadraticExpression):
-            string = glyphs.clever_sub(self.string, other.string)
-
-            quadPart = {}
-            varPairs = set(self._quads.keys()).union(other._quads.keys())
-            for vars in varPairs:
-                if vars in self._quads:
-                    if vars in other._quads:
-                        quadPart[vars] = self._quads[vars] - other._quads[vars]
-                    else:
-                        quadPart[vars] = self._quads[vars]
-                else:
-                    quadPart[vars] = -other._quads[vars]
-
-            affPart = self._aff - other._aff
-
-            return QuadraticExpression(string, quadPart, affPart)
-        elif isinstance(other, AffineExpression):
-            string = glyphs.clever_sub(self.string, other.string)
-
-            return QuadraticExpression(
-                string, self._quads, self._aff - other, copyDecomposition=self)
-        else:
-            return NotImplemented
+        return QuadraticExpression._add_sub(self, other, False, True)
 
     @convert_operands(sameShape=True)
     def __rsub__(self, other):
         """Denote substraction with self on the right hand side."""
-        if isinstance(other, QuadraticExpression):
-            assert False, "Impossible case."
-        elif isinstance(other, AffineExpression):
-            result = (-self).__add__(other)
-            # NOTE: __add__ always creates a fresh expression.
-            result._symbStr = glyphs.clever_sub(other.string, self.string)
-            return result
-        else:
-            return NotImplemented
+        return QuadraticExpression._add_sub(self, other, False, False)
 
     @cached_selfinverse_unary_operator
     def __neg__(self):
-        string   = glyphs.clever_neg(self.string)
-        quadPart = {xy: -A for xy, A in self._quads.items()}
-        affPart  = -self._aff
-        factors  = (self._sf[0], -self._sf[1]) if self._sf else None
+        string  = glyphs.clever_neg(self.string)
+        quads   = {xy: -A for xy, A in self._quads.items()}
+        affine  = -self._aff
+        factors = (self._sf[0], -self._sf[1]) if self._sf else None
 
-        return QuadraticExpression(string, quadPart, affPart, factors)
+        return QuadraticExpression(string, quads, affine, factors)
+
+    @classmethod
+    def _mul_div(cls, self, other, div, forward):
+        assert not div or forward
 
-    @convert_operands(scalarRHS=True)
-    def __mul__(self, other):
-        """Denote scaling from the right hand side."""
         if isinstance(other, AffineExpression):
             if not other.constant:
-                raise NotImplementedError("You may only multiply a PICOS "
-                    "quadratic expression with a constant term.")
+                if div:
+                    # NIE as this would makes sense if dividing by a factor.
+                    raise NotImplementedError("You may only divide a quadratic "
+                        "expression by a constant term.")
+                else:
+                    raise TypeError("You may only multiply a quadratic "
+                        "expression with a constant term.")
 
-            factor   = other.value
-            string   = glyphs.clever_mul(self.string, other.string)
-            quadPart = {xy: factor*A for xy, A in self._quads.items()}
-            affPart  = self._aff*other
+            factor = other.safe_value
+
+            if not factor:
+                if div:
+                    raise ZeroDivisionError(
+                        "Cannot divide {} by zero.".format(self.string))
+                else:
+                    return AffineExpression.zero()
+            elif factor == 1:
+                return self
+            elif factor == -1:
+                return -self
+
+            if div:
+                factor = 1 / factor
+                affine = self._aff / other
+                string = glyphs.div(self.string, other.string)
+            elif forward:
+                affine = self._aff * other
+                string = glyphs.clever_mul(self.string, other.string)
+            else:
+                affine = other * self._aff
+                string = glyphs.clever_mul(other.string, self.string)
+
+            quads = {xy: factor*A for xy, A in self._quads.items()}
 
             if self._sf:
                 r = factor**0.5
-                if self._sf[0] is self._sf[1]:
+                if r.imag:
+                    factors = (r.imag*self._sf[0], -r.imag*self._sf[1])
+                elif self._sf[0] is self._sf[1]:
                     factors = (r*self._sf[0],)*2
                 else:
                     factors = (r*self._sf[0], r*self._sf[1])
             else:
                 factors = None
 
-            return QuadraticExpression(string, quadPart, affPart, factors)
+            return cls(string, quads, affine, factors)
+
+        if div:
+            return Expression.__truediv__(self, other)
+        elif forward:
+            return Expression.__mul__(self, other)
         else:
-            return NotImplemented
+            return Expression.__rmul__(self, other)
+
+    @convert_operands(scalarRHS=True)
+    def __mul__(self, other):
+        """Denote scaling from the right hand side."""
+        return QuadraticExpression._mul_div(self, other, False, True)
 
     @convert_operands(scalarRHS=True)
     def __rmul__(self, other):
         """Denote scaling from the left hand side."""
-        if isinstance(other, AffineExpression):
-            result = self.__mul__(other)
-            # NOTE: __mul__ always creates a fresh expression.
-            result._symbStr = glyphs.clever_mul(other.string, self.string)
-            return result
-        else:
-            return NotImplemented
+        return QuadraticExpression._mul_div(self, other, False, False)
 
     @convert_operands(scalarRHS=True)
     def __truediv__(self, other):
         """Denote division by a constant scalar."""
-        if isinstance(other, AffineExpression):
-            if not other.constant:
-                raise NotImplementedError("You may only divide a PICOS "
-                    "quadratic expression by a constant term.")
-
-            result = self.__mul__(1.0 / other.value)
-            # NOTE: __mul__ always creates a fresh expression.
-            result._symbStr = glyphs.div(self.string, other.string)
-            return result
-        else:
-            return NotImplemented
+        return QuadraticExpression._mul_div(self, other, True, True)
 
     # --------------------------------------------------------------------------
     # Decomposition related methods.
