@@ -170,124 +170,120 @@ class SumExponentials(Expression):
     # Python special method implementations, except constraint-creating ones.
     # --------------------------------------------------------------------------
 
+    @classmethod
+    def _add(cls, self, other, forward):
+        if isinstance(other, AffineExpression) and other.constant:
+            value = other.value
+
+            if not value:
+                return self
+            elif value > 0:
+                if self._y is None:
+                    result = cls(self._x // math.log(value))
+                else:
+                    result = cls(self._x // value, self._y // 1)
+
+                if forward:
+                    string = glyphs.clever_add(self.string, other.string)
+                else:
+                    string = glyphs.clever_add(other.string, self.string)
+
+                result._typeStr = "Offset " + result._typeStr
+                result._symbStr = string
+
+                return result
+        elif isinstance(other, cls):
+            assert forward, "Encountered __radd__ on equal types."
+
+            if self._y is None and other._y is None:
+                result = cls(self._x.vec // other._x.vec)
+            elif self._y is not None and other._y is None:
+                one = AffineExpression.from_constant(1.0, (other.n, 1))
+                result = cls(self._x.vec // other._x.vec, self._y.vec // one)
+            elif self._y is None and other._y is not None:
+                one = AffineExpression.from_constant(1.0, (self.n, 1))
+                result = cls(self._x.vec // other._x.vec, one // other._y.vec)
+            else:
+                result = cls(
+                    self._x.vec // other._x.vec, self._y.vec // other._y.vec)
+
+            result._symbStr = glyphs.clever_add(self.string, other.string)
+
+            return result
+
+        if forward:
+            return Expression.__add__(self, other)
+        else:
+            return Expression.__radd__(self, other)
+
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __add__(self, other):
-        if isinstance(other, AffineExpression):
-            if not other.constant:
-                raise NotImplementedError("You may only add a constant term to "
-                    "a nonconstant PICOS sum of exponentials.")
-
-            value = other.value
-
-            if value < 0:
-                raise NotImplementedError("You may only add a nonnegative term "
-                    "to a nonconstant PICOS sum of exponentials.")
-
-            if value == 0:
-                # NOTE: We could return self here, but this is more consistent
-                #       with other expressions' __add__ methods.
-                sumexp = SumExponentials(self._x)
-            elif self._y is None:
-                sumexp = SumExponentials(self._x // math.log(value))
-            else:
-                sumexp = SumExponentials(self._x // value, self._y // 1)
-
-            sumexp._typeStr = "Offset " + sumexp._typeStr
-            sumexp._symbStr = glyphs.clever_add(self.string, other.string)
-
-            return sumexp
-        elif isinstance(other, SumExponentials):
-            if self._y is None and other._y is None:
-                sumexp = SumExponentials(self._x.vec // other._x.vec)
-            elif self._y is not None and other._y is None:
-                one = AffineExpression.from_constant(1.0, (other.n, 1))
-                sumexp = SumExponentials(
-                    self._x.vec // other._x.vec, self._y.vec // one)
-            elif self._y is None and other._y is not None:
-                one = AffineExpression.from_constant(1.0, (self.n, 1))
-                sumexp = SumExponentials(
-                    self._x.vec // other._x.vec, one // other._y.vec)
-            else:
-                sumexp = SumExponentials(
-                    self._x.vec // other._x.vec, self._y.vec // other._y.vec)
-
-            sumexp._symbStr = glyphs.clever_add(self.string, other.string)
-
-            return sumexp
-        else:
-            return NotImplemented
+        return SumExponentials._add(self, other, True)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __radd__(self, other):
-        if isinstance(other, (AffineExpression, SumExponentials)):
-            sumexp = self.__add__(other)
-            # NOTE: __add__ always creates a fresh expression.
-            sumexp._symbStr = glyphs.clever_add(other.string, self.string)
-            return sumexp
+        return SumExponentials._add(self, other, False)
+
+    @classmethod
+    def _mul_div(cls, self, other, div, forward):
+        assert not div or forward
+
+        if isinstance(other, AffineExpression) and other.constant:
+            factor = other.safe_value
+
+            if not factor:
+                if div:
+                    raise ZeroDivisionError(
+                        "Cannot divide {} by zero.".format(self.string))
+                else:
+                    return AffineExpression.zero()
+            elif factor == 1:
+                return self
+            elif factor > 0:
+                if div:
+                    factor = 1 / factor
+                    string = glyphs.div(self.string, other.string)
+                elif forward:
+                    string = glyphs.clever_mul(self.string, other.string)
+                else:
+                    string = glyphs.clever_mul(other.string, self.string)
+
+                if self._y is None:
+                    result = cls(self._x + math.log(factor))
+                else:
+                    result = cls(other*self._x, other*self._y)
+
+                result._typeStr = "Scaled " + result._typeStr
+                result._symbStr = string
+
+                return result
+
+        if div:
+            return Expression.__div__(self, other)
+        elif forward:
+            return Expression.__mul__(self, other)
         else:
-            return NotImplemented
+            return Expression.__rmul__(self, other)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __mul__(self, other):
-        if isinstance(other, AffineExpression):
-            if not other.constant:
-                raise NotImplementedError("You may only multiply a nonconstant "
-                    "PICOS sum of exponentials with a constant term.")
-
-            value = other.value
-
-            if value < 0:
-                raise NotImplementedError("You may only multiply a nonconstant "
-                    "PICOS sum of exponential with a nonnegative term.")
-
-            if value == 0:
-                return AffineExpression.zero()
-
-            if self._y is None:
-                sumexp = SumExponentials(self._x + math.log(value))
-            else:
-                sumexp = SumExponentials(self._x * value, self._y * value)
-
-            sumexp._typeStr = "Scaled " + sumexp._typeStr
-            sumexp._symbStr = glyphs.clever_mul(self.string, other.string)
-
-            return sumexp
-        else:
-            return NotImplemented
+        """Denote scaling from the right hand side."""
+        return SumExponentials._mul_div(self, other, div=False, forward=True)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __rmul__(self, other):
-        if isinstance(other, AffineExpression):
-            sumexp = self.__mul__(other)
-            # NOTE: __mul__ always creates a fresh expression.
-            sumexp._symbStr = glyphs.clever_mul(other.string, self.string)
-            return sumexp
-        else:
-            return NotImplemented
+        """Denote scaling from the left hand side."""
+        return SumExponentials._mul_div(self, other, div=False, forward=False)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __truediv__(self, other):
-        if isinstance(other, AffineExpression):
-            if not other.constant:
-                raise NotImplementedError("You may only divide a nonconstant "
-                    "PICOS sum of exponentials by a constant term.")
-
-            value = other.value
-
-            if value <= 0:
-                raise NotImplementedError("You may only divide a nonconstant "
-                    "PICOS sum of exponential by a positive term.")
-
-            sumexp = self * (1.0 / value)
-            # NOTE: __mul__ always creates a fresh expression.
-            sumexp._symbStr = glyphs.div(self.string, other.string)
-        else:
-            return NotImplemented
+        """Denote division by a constant scalar."""
+        return SumExponentials._mul_div(self, other, div=True, forward=True)
 
     # --------------------------------------------------------------------------
     # Methods and properties that return expressions.
