@@ -234,36 +234,42 @@ class SumExtremes(Expression):
     # Python special method implementations, except constraint-creating ones.
     # --------------------------------------------------------------------------
 
+    @classmethod
+    def _mul(cls, self, other, forward):
+        if isinstance(other, AffineExpression) and other.constant:
+            factor = other.safe_value
+
+            if not factor:
+                return AffineExpression.zero()
+            elif factor == 1:
+                return self
+            elif factor > 0:
+                if forward:
+                    string = glyphs.clever_mul(self.string, other.string)
+                else:
+                    string = glyphs.clever_mul(other.string, self.string)
+
+                product = cls(
+                    factor*self._x, self._k, self._largest, self._eigenvalues)
+                product._typeStr = "Scaled " + product._typeStr
+                product._symbStr = string
+
+                return product
+
+        if forward:
+            return Expression.__mul__(self, other)
+        else:
+            return Expression.__rmul__(self, other)
+
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __mul__(self, other):
-        if isinstance(other, AffineExpression):
-            if not other.constant:
-                raise NotImplementedError("You may only multiply a nonconstant "
-                    "PICOS sum of extremes with a constant term.")
-
-            if other.value < 0:
-                raise NotImplementedError("You may only multiply a nonconstant "
-                    "PICOS sum of extremes with a nonnegative term.")
-
-            product = SumExtremes(
-                other.value*self._x, self._k, self._largest, self._eigenvalues)
-            product._typeStr = "Scaled " + product._typeStr
-            product._symbStr = glyphs.clever_mul(self.string, other.string)
-            return product
-        else:
-            return NotImplemented
+        return SumExtremes._mul(self, other, True)
 
     @convert_operands(scalarRHS=True)
     @refine_operands()
     def __rmul__(self, other):
-        if isinstance(other, AffineExpression):
-            product = self.__mul__(other)
-            # NOTE: __mul__ always creates a fresh expression.
-            product._symbStr = glyphs.clever_mul(other.string, self.string)
-            return product
-        else:
-            return NotImplemented
+        return SumExtremes._mul(self, other, False)
 
     # --------------------------------------------------------------------------
     # Methods and properties that return expressions.
