@@ -23,17 +23,16 @@ import functools
 import operator
 import threading
 import warnings
-from abc import ABC, abstractmethod
+from abc import abstractmethod
 from contextlib import contextmanager
 
-import cvxopt
-
 from .. import glyphs
 from ..apidoc import api_end, api_start
 from ..caching import cached_property
 from ..constraints import ConstraintType
 from ..containers import DetailedType
 from ..legacy import deprecated
+from ..valuable import NotValued, Valuable
 from .data import convert_operands
 
 _API_START = api_start(globals())
@@ -186,17 +185,6 @@ def no_refinement():
         _REFINEMENT.allowed = True
 
 
-class NotValued(RuntimeError):
-    """The operation cannot be performed due to a mutable without a value.
-
-    Note that the :attr:`~.expression.Expression.value` and
-    :attr:`~.expression.Expression.value_as_matrix` attributes do not raise this
-    exception, but return :obj:`None`.
-    """
-
-    pass
-
-
 class PredictedFailure(TypeError):
     """Denotes that comparing two expressions will not form a constraint."""
 
@@ -282,7 +270,7 @@ class ExpressionType(DetailedType):
             return result
 
 
-class Expression(ABC):
+class Expression(Valuable):
     """Abstract base class for mathematical expressions, including mutables.
 
     For mutables, this is the secondary base class, with
@@ -312,6 +300,14 @@ class Expression(ABC):
         """
         return self._symbStr
 
+    # --------------------------------------------------------------------------
+    # Abstract method implementations for the Valuable base class.
+    # NOTE: _get_value and possibly _set_value are implemented by subclasses.
+    # --------------------------------------------------------------------------
+
+    def _get_valuable_string(self):
+        return "expression {}".format(self.string)
+
     # --------------------------------------------------------------------------
     # Abstract and default-implementation methods.
     # --------------------------------------------------------------------------
@@ -362,22 +358,6 @@ class Expression(ABC):
         """
         pass
 
-    @abstractmethod
-    def _get_value(self):
-        """Return the numeric value of the expression as a CVXOPT matrix.
-
-        :raises NotValued: When the value is not fully defined.
-
-        Method implementations need to return an independent copy of the value
-        that the user is allowed to change without affecting the expression.
-        """
-        pass
-
-    def _set_value(self, value):
-        raise NotImplementedError("Setting the value on an instance of {} is "
-            "not supported, but you can value any mutables involved instead."
-            .format(type(self).__name__))
-
     def _get_shape(self):
         """Return the algebraic shape of the expression."""
         return (1, 1)
@@ -529,177 +509,6 @@ class Expression(ABC):
         """Create a detailed expression type from subtype parameters."""
         return ExpressionType(cls, cls.Subtype(*args, **kwargs))
 
-    def _wrap_get_value(self, asMatrix, staySafe):
-        """Enhance the implementation of :attr:`_get_value`.
-
-        Checks the type of any value returned and offers conversion options.
-
-        :param bool asMatrix:
-            Whether scalar values are returned as matrices.
-
-        :param bool staySafe:
-            Whether :exc:`NotValued` exceptions are raised. Otherwise missing
-            values are returned as :obj:`None`.
-        """
-        try:
-            value = self._get_value()
-        except NotValued:
-            if staySafe:
-                raise
-            else:
-                return None
-
-        assert isinstance(value, (cvxopt.matrix, cvxopt.spmatrix)), \
-            "Expression._get_value implementations must return a CVXOPT matrix."
-
-        if value.size == (1, 1) and not asMatrix:
-            return value[0]
-        else:
-            return value
-
-    value = property(
-        lambda self: self._wrap_get_value(asMatrix=False, staySafe=False),
-        lambda self, x: self._set_value(x),
-        lambda self: self._set_value(None),
-        r"""Value of the expression, or :obj:`None`.
-
-        It is defined if the expression is constant or if all mutables involved
-        in the expression are valued. Mutables can be valued directly by writing
-        to their :attr:`value` attribute. Variables are also valued by PICOS
-        when an optimization solution is found.
-
-        Some expressions can also be valued directly if PICOS can find a minimal
-        norm mutable assignment that makes the expression have the desired
-        value. In particular, this works with affine expressions whose linear
-        part has an under- or well-determined coefficient matrix.
-
-        :returns:
-            The value as a Python scalar or CVXOPT matrix, or :obj:`None` if it
-            is not defined.
-
-        :Distinction:
-
-        - Unlike :attr:`safe_value` and :attr:`safe_value_as_matrix`, an
-          undefined value is returned as :obj:`None`.
-        - Unlike :attr:`value_as_matrix` and :attr:`safe_value_as_matrix`,
-          scalars are returned as scalar types.
-        - For uncertain expressions, see also
-          :meth:`~.uexpression.UncertainExpression.worst_case_value`.
-
-        :Example:
-
-        >>> from picos import RealVariable
-        >>> x = RealVariable("x", (1,3))
-        >>> y = RealVariable("y", (1,3))
-        >>> e = x - 2*y + 3
-        >>> print("e:", e)
-        e: x - 2·y + [3]
-        >>> e.value = [4, 5, 6]
-        >>> print("e: ", e, "\nx: ", x, "\ny: ", y, sep = "")
-        e: [ 4.00e+00  5.00e+00  6.00e+00]
-        x: [ 2.00e-01  4.00e-01  6.00e-01]
-        y: [-4.00e-01 -8.00e-01 -1.20e+00]
-        """)
-
-    safe_value = property(
-        lambda self: self._wrap_get_value(asMatrix=False, staySafe=True),
-        lambda self, x: self._set_value(x),
-        lambda self: self._set_value(None),
-        """Value of the expression, if defined.
-
-        Refer to :attr:`value` for when it is defined.
-
-        :returns:
-            The value as a Python scalar or CVXOPT matrix.
-
-        :raises ~picos.NotValued:
-            If the value is not defined.
-
-        :Distinction:
-
-        - Unlike :attr:`value`, an undefined value raises an exception.
-        - Like :attr:`value`, scalars are returned as scalar types.
-        """)
-
-    value_as_matrix = property(
-        lambda self: self._wrap_get_value(asMatrix=True, staySafe=False),
-        lambda self, x: self._set_value(x),
-        lambda self: self._set_value(None),
-        r"""Value of the expression as a CVXOPT matrix type, or :obj:`None`.
-
-        Refer to :attr:`value` for when it is defined (not :obj:`None`).
-
-        :returns:
-            The value as a CVXOPT matrix, or :obj:`None` if it is not defined.
-
-        :Distinction:
-
-        - Like :attr:`value`, an undefined value is returned as :obj:`None`.
-        - Unlike :attr:`value`, scalars are returned as :math:`1 \times 1`
-          matrices.
-        """)
-
-    safe_value_as_matrix = property(
-        lambda self: self._wrap_get_value(asMatrix=True, staySafe=True),
-        lambda self, x: self._set_value(x),
-        lambda self: self._set_value(None),
-        r"""Value of the expression as a CVXOPT matrix type, if defined.
-
-        Refer to :attr:`value` for when it is defined.
-
-        :returns:
-            The value as a CVXOPT matrix.
-
-        :raises ~picos.NotValued:
-            If the value is not defined.
-
-        :Distinction:
-
-        - Unlike :attr:`value`, an undefined value raises an exception.
-        - Unlike :attr:`value`, scalars are returned as :math:`1 \times 1`
-          matrices.
-        """)
-
-    @property
-    def valued(self):
-        """Whether the expression is valued.
-
-        .. note::
-
-            Querying this attribute is *not* faster than immediately querying
-            :attr:`value` and checking whether it is :obj:`None`. Use it only if
-            you do not need to know the value, but only whether it is available.
-
-        :Example:
-
-        >>> from picos import RealVariable
-        >>> x = RealVariable("x", 3)
-        >>> x.valued
-        False
-        >>> x.value
-        >>> print((x|1))
-        ∑(x)
-        >>> x.value = [1, 2, 3]
-        >>> (x|1).valued
-        True
-        >>> print((x|1))
-        6.0
-        """
-        try:
-            self._get_value()
-        except NotValued:
-            return False
-        else:
-            return True
-
-    @valued.setter
-    def valued(self, x):
-        if x is False:
-            self._set_value(None)
-        else:
-            raise ValueError("You may only assign 'False' to the 'valued' "
-                "attribute, which is the same as setting 'value' to 'None'.")
-
     shape = property(
         lambda self: self._get_shape(),
         doc=_get_shape.__doc__)
@@ -1041,7 +850,8 @@ class Expression(ABC):
 
         from .exp_affine import Constant
 
-        A = Constant(glyphs.frozen(self.string), self._get_value(), self.shape)
+        A = Constant(
+            glyphs.frozen(self.string), self.safe_value_as_matrix, self.shape)
         A._origin = self
         return A
 
@@ -1064,182 +874,175 @@ class Expression(ABC):
         valued, then a string representation of the value is returned.
         Otherwise, the symbolic description of the expression is returned.
         """
-        if self.valued:
-            return str(self.value).strip()
-        else:
+        value = self.value
+
+        if value is None:
             return str(self._symbStr)
+        else:
+            return str(value).strip()
 
     def __format__(self, format_spec):
-        if self.valued:
-            return self.value.__format__(format_spec)
-        else:
+        value = self.value
+
+        if value is None:
             return self._symbStr.__format__(format_spec)
-
-    def __index__(self):
-        if len(self) != 1:
-            raise TypeError("Cannot use multidimensional expression {} as an "
-                "index.".format(self.string))
-
-        if not self.valued:
-            raise NotValued("Cannot use unvalued expression {} as an index."
-                .format(self.string))
-
-        value = self.safe_value
-
-        if value.imag:
-            raise ValueError("Cannot use {} as an index as its value of {} has "
-                "a nonzero imaginary part.".format(self.string, value))
-
-        value = value.real
-
-        if not value.is_integer():
-            raise ValueError("Cannot use {} as an index as its value of {} is "
-                "not integral.".format(self.string, value))
-
-        return int(value)
-
-    def _casting_helper(self, theType):
-        assert theType in (int, float, complex)
-
-        if len(self) != 1:
-            raise TypeError(
-                "Cannot cast multidimensional expression {} as {}."
-                .format(self.string, theType.__name__))
-
-        if not self.valued:
-            raise NotValued("Cannot cast unvalued expression {} as {}."
-                .format(self.string, theType.__name__))
-
-        value = self.value_as_matrix
-
-        return theType(value[0])
-
-    def __int__(self):
-        return self._casting_helper(int)
-
-    def __float__(self):
-        return self._casting_helper(float)
-
-    def __complex__(self):
-        return self._casting_helper(complex)
-
-    def __round__(self, ndigits=None):
-        return round(float(self), ndigits)
+        else:
+            return value.__format__(format_spec)
 
     # Since we define __eq__, __hash__ is not inherited. Do this manually.
     __hash__ = object.__hash__
 
-    # HACK: This prevents NumPy operators from iterating over PICOS expressions.
-    __array_priority__ = float("inf")
-
     # --------------------------------------------------------------------------
     # Fallback algebraic operations: Try again with converted RHS, refined LHS.
+    # NOTE: The forward operations call the backward operations manually
+    #       (instead of returning NotImplemented) so that they can be performed
+    #       on a converted operand, which is always a PICOS type. The backward
+    #       operations then use WeightedSum as a last fallback where applicable.
     # --------------------------------------------------------------------------
 
+    def _wsum_fallback(self, summands, weights, opstring):
+        """Try to represent the result as a weighted sum."""
+        from .exp_wsum import WeightedSum
+
+        # NOTE: WeightedSum with an opstring set will act as a final fallback
+        #       and raise a proper exception if the result can't be represented.
+        #       This is handled there and not here so that also operations on
+        #       existing WeightedSum instances can produce such exceptions, as
+        #       they cannot fallback to Expression like other operations do.
+        return WeightedSum(summands, weights, opstring)
+
+    def _scalar_mult_fallback(self, lhs, rhs):
+        """Try to express scalar by scalar multiplication as a weighted sum."""
+        assert isinstance(lhs, Expression) and isinstance(rhs, Expression)
+
+        opstring = "a product between {} and {}".format(repr(lhs), repr(rhs))
+
+        if lhs.scalar and lhs.constant:
+            return self._wsum_fallback((rhs,), lhs.safe_value, opstring)
+        elif rhs.scalar and rhs.constant:
+            return self._wsum_fallback((lhs,), rhs.safe_value, opstring)
+        else:
+            # NOTE: Constant scalars are also AffineExpression but otherwise
+            #       raising the default Python TypeError (stating that the two
+            #       types are fully operation-incompatible) makes sense here.
+            return NotImplemented
+
     @convert_operands(sameShape=True)
     def __add__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__add__(other)
         else:
-            return NotImplemented
+            return other.__radd__(self)
 
     @convert_operands(sameShape=True)
     def __radd__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__radd__(other)
         else:
-            return NotImplemented
+            opstring = "{} plus {}".format(repr(other), repr(self))
+            return self._wsum_fallback((other, self), (1, 1), opstring)
 
     @convert_operands(sameShape=True)
     def __sub__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__sub__(other)
         else:
-            return NotImplemented
+            return other.__rsub__(self)
 
     @convert_operands(sameShape=True)
     def __rsub__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__rsub__(other)
         else:
-            return NotImplemented
+            opstring = "{} minus {}".format(repr(other), repr(self))
+            return self._wsum_fallback((other, self), (1, -1), opstring)
 
     @convert_operands(sameShape=True)
     def __or__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__or__(other)
         else:
-            return NotImplemented
+            return other.__ror__(self)
 
     @convert_operands(sameShape=True)
     def __ror__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__ror__(other)
         else:
-            return NotImplemented
+            return self._scalar_mult_fallback(other, self)
 
     @convert_operands(rMatMul=True)
     def __mul__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__mul__(other)
         else:
-            return NotImplemented
+            return other.__rmul__(self)
 
     @convert_operands(lMatMul=True)
     def __rmul__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__rmul__(other)
         else:
-            return NotImplemented
+            return self._scalar_mult_fallback(other, self)
 
     @convert_operands(sameShape=True)
     def __xor__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__xor__(other)
         else:
-            return NotImplemented
+            return other.__rxor__(self)
 
     @convert_operands(sameShape=True)
     def __rxor__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__rxor__(other)
         else:
-            return NotImplemented
+            return self._scalar_mult_fallback(other, self)
 
     @convert_operands()
     def __matmul__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__matmul__(other)
         else:
-            return NotImplemented
+            return other.__rmatmul__(self)
 
     @convert_operands()
     def __rmatmul__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__rmatmul__(other)
         else:
-            return NotImplemented
+            return self._scalar_mult_fallback(other, self)
 
     @convert_operands(scalarRHS=True)
     def __truediv__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__truediv__(other)
         else:
-            return NotImplemented
+            return other.__rtruediv__(self)
 
     @convert_operands(scalarLHS=True)
     def __rtruediv__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__rtruediv__(other)
         else:
-            return NotImplemented
+            if self.constant and not self.is0:
+                try:
+                    return other.__mul__(1 / self.safe_value)
+                except TypeError:
+                    assert False, "Multiplication of {} by a nonzero constant" \
+                        " has unexpectedly failed; it should have produced a " \
+                        "weighted sum.".format(repr(other))
+            else:
+                reason = "nonconstant" if not self.constant else "zero"
+                raise TypeError("Cannot divide {} by {}: The denominator is {}."
+                    .format(repr(other), repr(self), reason))
 
     @convert_operands(scalarRHS=True)
     def __pow__(self, other):
         if type(self.refined) != type(self):
             return self.refined.__pow__(other)
         else:
-            return NotImplemented
+            return other.__rpow__(self)
 
     @convert_operands(scalarLHS=True)
     def __rpow__(self, other):
@@ -1253,7 +1056,7 @@ class Expression(ABC):
         if type(self.refined) != type(self):
             return self.refined.__and__(other)
         else:
-            return NotImplemented
+            return other.__rand__(self)
 
     @convert_operands(horiCat=True)
     def __rand__(self, other):
@@ -1267,7 +1070,7 @@ class Expression(ABC):
         if type(self.refined) != type(self):
             return self.refined.__floordiv__(other)
         else:
-            return NotImplemented
+            return other.__rfloordiv__(self)
 
     @convert_operands(vertCat=True)
     def __rfloordiv__(self, other):
@@ -1276,11 +1079,15 @@ class Expression(ABC):
         else:
             return NotImplemented
 
+    def __pos__(self):
+        return self
+
     def __neg__(self):
         if type(self.refined) != type(self):
             return self.refined.__neg__()
         else:
-            return NotImplemented
+            opstring = "the negation of {}".format(repr(self))
+            return self._wsum_fallback((self,), -1, opstring)
 
     def __abs__(self):
         if type(self.refined) != type(self):
@@ -1299,7 +1106,7 @@ class Expression(ABC):
     def _rshift_implementation(self, other):
         return NotImplemented
 
-    @convert_operands(sameShape=True)
+    @convert_operands(diagBroadcast=True)
     @validate_prediction
     @refine_operands()
     def __lshift__(self, other):
@@ -1310,7 +1117,7 @@ class Expression(ABC):
 
         return result
 
-    @convert_operands(sameShape=True)
+    @convert_operands(diagBroadcast=True)
     @validate_prediction
     @refine_operands()
     def __rshift__(self, other):
@@ -1325,12 +1132,12 @@ class Expression(ABC):
     # Backwards compatibility methods.
     # --------------------------------------------------------------------------
 
-    @deprecated("2.0", useInstead="valued")
+    @deprecated("2.0", useInstead="~picos.valuable.Valuable.valued")
     def is_valued(self):
         """Whether the expression is valued."""
         return self.valued
 
-    @deprecated("2.0", useInstead="value")
+    @deprecated("2.0", useInstead="~picos.valuable.Valuable.value")
     def set_value(self, value):
         """Set the value of an expression."""
         self.value = value
