@@ -18,6 +18,8 @@
 
 """Implements :class:`Samples`."""
 
+import random
+
 import cvxopt
 
 from .. import glyphs
@@ -365,6 +367,44 @@ class Samples():
 
         return Constant(Z * Z.T / (self.num - 1))
 
+    def shuffled(self, rng=None):
+        """Return a randomly shuffled instance of the samples.
+
+        :param rng:
+            A function that generates a random :class:`float` in :math:`[0, 1)`.
+            Defaults to whatever :func:`random.shuffle` defaults to.
+
+        :Example:
+
+        >>> from picos.expressions import Samples
+        >>> S = Samples(range(6))
+        >>> print(S.matrix)
+        [ 0.00e+00  1.00e+00  2.00e+00  3.00e+00  4.00e+00  5.00e+00]
+        >>> rng = lambda: 0.5  # Fake RNG for reproducibility.
+        >>> print(S.shuffled(rng).matrix)
+        [ 0.00e+00  5.00e+00  1.00e+00  4.00e+00  2.00e+00  3.00e+00]
+        """
+        order = list(range(self.num))
+        random.shuffle(order, rng)
+
+        S = self.__class__.__new__(self.__class__)
+
+        if self._cached_cvx_mat is not None:
+            S._cached_cvx_mat = self._cached_cvx_mat[:, order]
+
+        if self._cached_cvx_vec is not None:
+            S._cached_cvx_vec = tuple(self._cached_cvx_vec[i] for i in order)
+
+        if self._cached_pic_mat is not None:
+            # NOTE: Rename to a default string for consistency.
+            S._cached_pic_mat = self._cached_pic_mat[:, order].renamed(
+                glyphs.matrix(glyphs.shape(self._cached_pic_mat.shape)))
+
+        if self._cached_pic_vec is not None:
+            S._cached_pic_vec = tuple(self._cached_pic_vec[i] for i in order)
+
+        return S
+
     def partition(self, after_or_fraction=0.5):
         """Split the samples into two parts.
 
@@ -413,6 +453,87 @@ class Samples():
 
         return A, B
 
+    def kfold(self, k):
+        r"""Perform :math:`k`-fold cross-validation (without shuffling).
+
+        If random shuffling is desired, write ``S.shuffled().kfold(k)`` where
+        ``S`` is your :class:`Samples` instance. To make the shuffling
+        reproducible, see :meth:`shuffled`.
+
+        :returns list(tuple):
+            A list of :math:`k` training set and validation set pairs.
+
+        .. warning::
+
+            If the number of samples :math:`n` is not a multiple of :math:`k`,
+            then the last :math:`n \bmod k` samples will appear in every
+            training but in no validation set.
+
+        :Example:
+
+        >>> from picos.expressions import Samples
+        >>> n, k = 7, 3
+        >>> S = Samples(range(n))
+        >>> for i, (T, V) in enumerate(S.kfold(k)):
+        ...     print("Partition {}:\nT = {}V = {}"
+        ...           .format(i + 1, T.matrix, V.matrix))
+        Partition 1:
+        T = [ 2.00e+00  3.00e+00  4.00e+00  5.00e+00  6.00e+00]
+        V = [ 0.00e+00  1.00e+00]
+        <BLANKLINE>
+        Partition 2:
+        T = [ 0.00e+00  1.00e+00  4.00e+00  5.00e+00  6.00e+00]
+        V = [ 2.00e+00  3.00e+00]
+        <BLANKLINE>
+        Partition 3:
+        T = [ 0.00e+00  1.00e+00  2.00e+00  3.00e+00  6.00e+00]
+        V = [ 4.00e+00  5.00e+00]
+        <BLANKLINE>
+
+        """
+        if not isinstance(k, int):
+            raise TypeError("k must be an integer.")
+
+        if k < 2:
+            raise ValueError("k must be at least two.")
+
+        if k > self.num:
+            raise ValueError("k must not exceed the number of samples.")
+
+        n = self.num // k
+
+        assert n >= 1 and n < self.num
+
+        fold = []
+        indices = list(range(self.num))
+
+        for i in range(k):
+            t = indices[:i*n] + indices[(i+1)*n:]
+            v = indices[i*n:(i+1)*n]
+
+            T = self.__class__.__new__(self.__class__)
+            V = self.__class__.__new__(self.__class__)
+
+            if self._cached_cvx_mat is not None:
+                T._cached_cvx_mat = self._cached_cvx_mat[:, t]
+                V._cached_cvx_mat = self._cached_cvx_mat[:, v]
+
+            if self._cached_cvx_vec is not None:
+                T._cached_cvx_vec = tuple(self._cached_cvx_vec[i] for i in t)
+                V._cached_cvx_vec = tuple(self._cached_cvx_vec[i] for i in v)
+
+            if self._cached_pic_mat is not None:
+                T._cached_pic_mat = self._cached_pic_mat[:, t]
+                V._cached_pic_mat = self._cached_pic_mat[:, v]
+
+            if self._cached_pic_vec is not None:
+                T._cached_pic_vec = tuple(self._cached_pic_vec[i] for i in t)
+                V._cached_pic_vec = tuple(self._cached_pic_vec[i] for i in v)
+
+            fold.append((T, V))
+
+        return fold
+
     def select(self, indices):
         """Return a new :class:`Samples` instance with only selected samples.
 
