@@ -1,41 +1,35 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019 Maximilian Stahlberg
 # Based on the original picos.expressions module by Guillaume Sagnol.
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
-"""Implements all mathematical variable types."""
+"""Implements all mathematical variable types and their base class."""
 
-import random
-import threading
 from collections import namedtuple
-from copy import copy
 
 import cvxopt
 
-from .. import glyphs
+from .. import glyphs, settings
 from ..apidoc import api_end, api_start
 from ..caching import cached_property
 from ..containers import DetailedType
-from .data import cvxopt_equals, cvxopt_maxdiff, load_data, load_shape
+from .data import cvxopt_equals, cvxopt_maxdiff, load_shape
 from .exp_affine import AffineExpression, ComplexAffineExpression
-from .expression import Expression, NotValued
+from .mutable import Mutable
 from .vectorizations import (ComplexVectorization, FullVectorization,
                              HermitianVectorization,
                              LowerTriangularVectorization,
@@ -47,50 +41,13 @@ _API_START = api_start(globals())
 # -------------------------------
 
 
-# The variable IDs start at a random value to prevent a clash if variables are
-# pickled and loaded in another python session. The choice of 31 bits is to
-# produce up to 2**31-1 short hexadecimal IDs for use in private variable names.
-_NEXT_VARIABLE_ID = int(random.getrandbits(31))
-
-# A lock for _NEXT_VARIABLE_ID, if the user uses threads to create variables.
-_VARIABLE_ID_LOCK = threading.Lock()
-
-
-def _make_variable_id(dim):
-    """Create a unique (starting) ID for a new variable."""
-    with _VARIABLE_ID_LOCK:
-        global _NEXT_VARIABLE_ID
-        id = _NEXT_VARIABLE_ID
-        _NEXT_VARIABLE_ID += dim
-        return id
-
-
-# A map used when converting IDs to strings.
-_ID_HEX2STR_MAP = {ord(a): ord(b) for a, b in zip(
-    "0123456789abcdef", "ABCDEFGHIJKLMNOP")}
-_ID_HEX2STR_MAP_PY2 = "".join([chr(_ID_HEX2STR_MAP[i]) if i in _ID_HEX2STR_MAP
-    else "_" for i in range(256)])
-
-
-def _id_to_str(id):
-    """Convert a variable id to a short string identifier."""
-    try:
-        return "{:08x}".format(id).translate(_ID_HEX2STR_MAP)
-    except TypeError:  # Python 2
-        return "{:08x}".format(id).translate(_ID_HEX2STR_MAP_PY2)
-
-
-#: Absolute integrality tolerance for integral and binary variables.
-INTEGRALITY_ABS_TOL = 1e-4
-
-
 class VariableType(DetailedType):
     """The detailed type of a variable for predicting reformulation outcomes."""
 
     pass
 
 
-class BaseVariable:
+class BaseVariable(Mutable):
     """Primary base class for all variable types.
 
     Variables need to inherit this class with priority (first class listed) and
@@ -120,26 +77,19 @@ class BaseVariable:
             Constant upper bound on the variable. May contain ``float("inf")``
             to denote unbounded elements.
         """
-        if not isinstance(self, Expression):
-            raise TypeError("{} may not be initialized directly.".format(
-                type(self).__name__))
+        Mutable.__init__(self, name, vectorization)
 
-        self._id    = _make_variable_id(vectorization.dim)
-        self._vec   = vectorization
         self._lower = None if lower is None else self._load_vectorized(lower)
         self._upper = None if upper is None else self._load_vectorized(upper)
-        self._value = None
 
-        if not name or name.startswith("__"):
-            id_str = "__{}_".format(_id_to_str(self._id))
-            name = name.replace("__", id_str, 1) if name else id_str + "_"
+    def copy(self, new_name=None):
+        """Return an independent copy of the variable."""
+        name = self.name if new_name is None else new_name
 
-        self._name = name
-
-    @property
-    def id(self):
-        """The unique (starting) ID of the variable, assigned at creation."""
-        return self._id
+        if self._lower is not None or self._upper is not None:
+            return self.__class__(name, self.shape, self._lower, self._upper)
+        else:
+            return self.__class__(name, self.shape)
 
     VarSubtype = namedtuple("VarSubtype", ("dim", "bnd"))
 
@@ -170,51 +120,20 @@ class BaseVariable:
         """
         return VariableType(self.__class__, self.var_subtype)
 
-    def id_at(self, index):
-        """Return the unique ID of a scalar entry, assigned at creation."""
-        if index < 0 or index >= self._vec.dim:
-            raise IndexError("Bad index {} for the {} dimensional variable {}."
-                .format(index, self._vec.dim, self.name))
-
-        return self._id + index
-
-    @property
-    def dim(self):
-        """The variable's dimension on the real field.
-
-        This corresponds to the length of its vectorized value.
-        """
-        return self._vec.dim
-
-    @property
-    def name(self):
-        """The name of the variable."""
-        return self._name
-
     @cached_property
     def long_string(self):
         """Long string representation for printing a :meth:`~picos.Problem`."""
         lower, upper = self.bound_dicts
         if lower and upper:
-            bound_str = "clamped"
+            bound_str = " (clamped)"
         elif lower:
-            bound_str = "bounded below"
+            bound_str = " (bounded below)"
         elif upper:
-            bound_str = "bounded above"
+            bound_str = " (bounded above)"
         else:
-            bound_str = None
+            bound_str = ""
 
-        return "{} {} {}{}".format(
-            glyphs.shape(self.shape),
-            self._get_type_string_base().lower(),
-            self.name,
-            " ({})".format(bound_str) if bound_str else "")
-
-        return "{} {} variable {}{}".format(
-            self._get_type_string_base().rsplit(" Variable")[0].lower(),
-            glyphs.shape(self.shape),
-            self.name,
-            " ({})".format(bound_str) if bound_str else "")
+        return "{}{}".format(super(BaseVariable, self).long_string, bound_str)
 
     @cached_property
     def bound_dicts(self):
@@ -222,7 +141,7 @@ class BaseVariable:
 
         The indices and bound values are with respect to the internal
         representation of the variable, whose value can be accessed with
-        :attr:`internal_value`.
+        :attr:`~.mutable.Mutable.internal_value`.
 
         Upper and lower bounds set to ``float("inf")`` and ``float("-inf")``
         on variable creation, respectively, are not included.
@@ -281,83 +200,6 @@ class BaseVariable:
 
         return Ax <= b
 
-    def copy(self, new_name=None):
-        """Return an independent copy of the variable."""
-        name = self.name if new_name is None else new_name
-
-        if self._lower is not None or self._upper is not None:
-            return self.__class__(name, self.shape, self._lower, self._upper)
-        else:
-            return self.__class__(name, self.shape)
-
-    def _load_vectorized(self, value):
-        """Support :meth:`__init__` and ``_set_value``."""
-        return self._vec.vectorize(
-            load_data(value, self._vec.shape, self._typecode)[0])
-
-    def _check_internal_value(self, value):
-        """Support :meth:`_set_value` and :meth:`_set_internal_value`."""
-        pass
-
-    # NOTE: This needs to be inherited with priority.
-    def _get_value(self):
-        return self._vec.devectorize(self._get_internal_value())
-
-    # NOTE: This needs to be inherited with priority.
-    def _set_value(self, value):
-        if value is None:
-            self._value = None
-            return
-
-        try:
-            value = self._load_vectorized(value)
-            self._check_internal_value(value)
-        except Exception as error:
-            raise type(error)("Failed to assign a value to variable {}: {}"
-                .format(self.string, error))
-        else:
-            self._value = value
-
-    def _get_internal_value(self):
-        if self._value is None:
-            raise NotValued("Variable {} is not valued.".format(self.string))
-        else:
-            return copy(self._value)
-
-    def _set_internal_value(self, value):
-        if value is None:
-            self._value = None
-            return
-
-        try:
-            value = load_data(value, (self._vec.dim, 1), "d")[0]
-            self._check_internal_value(value)
-        except Exception as error:
-            raise type(error)(
-                "Failed to assign an internal value to variable {}: {}"
-                .format(self.string, error))
-        else:
-            self._value = value
-
-    internal_value = property(
-        lambda self: self._get_internal_value(),
-        lambda self, value: self._set_internal_value(value),
-        lambda self: self._set_internal_value(None),
-        """The internal (special vectorized) value of the variable.""")
-
-    def _get_clstype(self):
-        # Mimic an ordinary (complex) affine expression.
-        return self._basetype
-
-    def _get_subtype(self):
-        # Mimic an ordinary (complex) affine expression.
-        return self._basetype._get_subtype(self)
-
-    @classmethod
-    def _predict(cls, subtype, relation, other):
-        # Mimic an ordinary (complex) affine expression.
-        return cls._get_basetype()._predict(subtype, relation, other)
-
 
 class RealVariable(BaseVariable, AffineExpression):
     """A real-valued variable."""
@@ -417,10 +259,12 @@ class IntegerVariable(BaseVariable, AffineExpression):
         intData = cvxopt.matrix([round(x) for x in fltData])
         fltData = cvxopt.matrix(fltData)
 
-        if not cvxopt_equals(intData, fltData, INTEGRALITY_ABS_TOL):
+        if not cvxopt_equals(intData, fltData,
+                absTol=settings.ABSOLUTE_INTEGRALITY_TOLERANCE):
             raise ValueError("Data is not near-integral with absolute tolerance"
                 " {:.1e}: Largest difference is {:.1e}.".format(
-                INTEGRALITY_ABS_TOL, cvxopt_maxdiff(intData, fltData)))
+                settings.ABSOLUTE_INTEGRALITY_TOLERANCE,
+                cvxopt_maxdiff(intData, fltData)))
 
 
 class BinaryVariable(BaseVariable, AffineExpression):
@@ -452,10 +296,12 @@ class BinaryVariable(BaseVariable, AffineExpression):
         binData = cvxopt.matrix([float(bool(round(x))) for x in fltData])
         fltData = cvxopt.matrix(fltData)
 
-        if not cvxopt_equals(binData, fltData, INTEGRALITY_ABS_TOL):
+        if not cvxopt_equals(binData, fltData,
+                absTol=settings.ABSOLUTE_INTEGRALITY_TOLERANCE):
             raise ValueError("Data is not near-binary with absolute tolerance"
                 " {:.1e}: Largest difference is {:.1e}.".format(
-                INTEGRALITY_ABS_TOL, cvxopt_maxdiff(binData, fltData)))
+                settings.ABSOLUTE_INTEGRALITY_TOLERANCE,
+                cvxopt_maxdiff(binData, fltData)))
 
 
 class ComplexVariable(BaseVariable, ComplexAffineExpression):
