@@ -1,39 +1,36 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019 Maximilian Stahlberg
 # Based on the original picos.expressions module by Guillaume Sagnol.
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 r"""Implement special matrix vectorization formats.
 
-These formats are used to efficiently store structured variable types such as
-symmetric matrix variables in the form of real variable vectors.
+These formats are used to efficiently store structured mutable types such as
+symmetric matrix variables in the form of real vectors.
 """
 
 import random
+from abc import ABC, abstractmethod
 from copy import copy
 
 import cvxopt
 
-from .. import glyphs
+from .. import glyphs, settings
 from ..apidoc import api_end, api_start
-from ..compat import ABC, abstractmethod
 from .data import cvxopt_equals, cvxopt_hcat, cvxopt_vcat, load_shape
 
 _API_START = api_start(globals())
@@ -98,7 +95,7 @@ class BaseVectorization(ABC, object):
     def dim(self):
         """The length of the vectorization.
 
-        This corresponds to the dimension of a matrix variable being vectorized.
+        This corresponds to the dimension of a matrix mutable being vectorized.
         """
         return self._special2full.size[1]
 
@@ -137,7 +134,7 @@ class BaseVectorization(ABC, object):
         if vector.size != (self.dim, 1):
             raise TypeError(
                 "Invalid shape of vectorized data: Expected {} but got {}."
-                .format(glyphs.shape((self.dim, 1), vector.size)))
+                .format(glyphs.shape((self.dim, 1)), glyphs.shape(vector.size)))
 
     @abstractmethod
     def _make_special_to_full(self):
@@ -159,8 +156,9 @@ class BaseVectorization(ABC, object):
 
         The term *identity* comes from the fact that these matrices are used
         as the coefficients that map the internal (vectorized) representation of
-        a :class:`~.variables.BaseVariable` object to the
-        :class:`~.exp_affine.ComplexAffineExpression` that it represents.
+        a :class:`~.mutable.Mutable` object to the
+        :class:`~.exp_biaffine.BiaffineExpression` instance that represents the
+        mutable in algebraic operations.
         """
         return self._special2full
 
@@ -274,7 +272,8 @@ class SymmetricVectorization(BaseVectorization):
         BaseVectorization._validate_matrix(self, matrix)
         self._ensure_real(matrix)
 
-        if not cvxopt_equals(matrix, matrix.T):
+        if not cvxopt_equals(matrix, matrix.T,
+                relTol=settings.RELATIVE_HERMITIANNESS_TOLERANCE):
             raise ValueError("The given matrix is not numerically symmetric.")
 
     def _make_full_to_special(self):
@@ -327,7 +326,9 @@ class SkewSymmetricVectorization(BaseVectorization):
         BaseVectorization._validate_matrix(self, matrix)
         self._ensure_real(matrix)
 
-        if not cvxopt_equals(matrix, -matrix.T):
+        # NOTE: Use hermitianess tolerance.
+        if not cvxopt_equals(matrix, -matrix.T,
+                relTol=settings.RELATIVE_HERMITIANNESS_TOLERANCE):
             raise ValueError(
                 "The given matrix is not numerically skew-symmetric.")
 
@@ -386,7 +387,8 @@ class HermitianVectorization(BaseVectorization):
     def _validate_matrix(self, matrix):
         BaseVectorization._validate_matrix(self, matrix)
 
-        if not cvxopt_equals(matrix, matrix.H):
+        if not cvxopt_equals(matrix, matrix.H,
+                relTol=settings.RELATIVE_HERMITIANNESS_TOLERANCE):
             raise ValueError("The given matrix is not numerically hermitian.")
 
     def _make_full_to_special(self):
