@@ -1,23 +1,20 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2012-2017 Guillaume Sagnol
 # Copyright (C)      2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Functions for writing optimization problems to a file."""
@@ -147,11 +144,11 @@ def write(picos_problem, filename, writer="picos"):
         else:
             try:
                 P = picos_problem.prepared(solver=None)
-            except NoStrategyFound:
+            except NoStrategyFound as error:
                 # FIXME: This will typically happen for integer linear programs
-                # if no IP solver is available.
-                raise NotImplementedError("You try to export a problem for "
-                    "which no solver is available")
+                #        if no IP solver is available.
+                raise RuntimeError("You try to export a problem for which no "
+                    "solver is available") from error
 
             assert 'Linear' in P.type, "only LINEAR integer problems can be " \
                 "exported with the default writer"
