@@ -439,6 +439,13 @@ OPTIONS = [
         See the `list of MOSEK (Fusion) 8.1 parameters
         <https://docs.mosek.com/8.1/pythonfusion/parameters.html>`_."""),
 
+    ("osqp_params", dict, {}, """
+        A dictionary of OSQP parameters to be set after general options are
+        passed and before the search is started.
+
+        See the `list of OSQP parameters
+        <https://osqp.org/docs/interfaces/solver_settings.html>`_."""),
+
     ("scip_params", dict, {}, """
         A dictionary of SCIP parameters to be set after general options are
         passed and before the search is started.
