@@ -38,6 +38,7 @@ from ..expressions.variables import BaseVariable
 from ..formatting import natsorted, parameterized_string, picos_box
 from ..legacy import deprecated, map_legacy_options, throw_deprecation_warning
 from ..solvers import Solver, get_solver
+from ..valuable import Valuable
 from .file_out import write
 from .footprint import Footprint, Specification
 from .objective import Objective
@@ -70,19 +71,19 @@ class SolutionFailure(RuntimeError):
         return "Code {}: {}".format(self.code, self.message)
 
 
-class Problem():
+class Problem(Valuable):
     """PICOS' representation of an optimization problem.
 
     :Example:
 
     >>> from picos import Problem, RealVariable
     >>> X = RealVariable("X", (2,2), lower = 0)
-    >>> P = Problem()
-    >>> P.set_objective("max", X.tr)
-    >>> C1 = P.add_constraint(X.sum <= 10)
-    >>> C2 = P.add_constraint(X[0,0] == 1)
+    >>> P = Problem("Example")
+    >>> P.maximize = X.tr
+    >>> C = X.sum <= 10
+    >>> P += C, X[0,0] == 1
     >>> print(P)
-    Linear Program
+    Example (Linear Program)
       maximize tr(X)
       over
         2×2 real variable X (bounded below)
@@ -100,7 +101,7 @@ class Problem():
     >>> print(X) #doctest: +SKIP
     [ 1.00e+00  4.89e-10]
     [ 4.89e-10  9.00e+00]
-    >>> round(C1.dual, 1)
+    >>> round(C.dual, 1)
     1.0
     """
 
@@ -116,9 +117,15 @@ class Problem():
     # Initialization and reset methods.
     # --------------------------------------------------------------------------
 
-    def __init__(self, copyOptions=None, useOptions=None, **extra_options):
+    def __init__(
+            self, name=None, *, copyOptions=None, useOptions=None,
+            **extra_options
+    ):
         """Create an empty problem and optionally set initial solver options.
 
+        :param str name:
+            A name or title to give to the optimization problem.
+
         :param copyOptions:
             An :class:`Options <picos.Options>` object to copy instead of using
             the default options.
@@ -130,6 +137,11 @@ class Problem():
             A sequence of additional solver options to apply on top of the
             default options or those given by ``copyOptions`` or ``useOptions``.
         """
+        if name and not isinstance(name, str):
+            raise TypeError(
+                "The first positional argument denotes the name of the problem "
+                "and must be a string.")
+
         if copyOptions and useOptions:
             raise ValueError(
                 "Can only copy or use existing solver options, not both.")
@@ -145,6 +157,9 @@ class Problem():
         else:
             self._options = Options(**extra_options)
 
+        #: Explicit name for the problem.
+        self._name = name
+
         #: The optimization objective.
         self._objective = Objective()
 
@@ -206,6 +221,25 @@ class Problem():
     # Properties.
     # --------------------------------------------------------------------------
 
+    @property
+    def name(self):
+        """Name or title of the problem."""
+        return self._name
+
+    @name.setter
+    def name(self, value):
+        if value and not isinstance(value, str):
+            raise TypeError("The problem name must be a string.")
+
+        if not value:
+            self._name = None
+        else:
+            self._name = value
+
+    @name.deleter
+    def name(self):
+        self._name = None
+
     @property
     def mutables(self):
         """Maps names to variables and parameters in use by the problem.
@@ -337,32 +371,6 @@ class Problem():
         else:
             raise ValueError("Objective direction is not maximize.")
 
-    @property
-    def value(self):
-        """Objective function value.
-
-        If all mutables that appear in the objective function are valued, in
-        particular after a successful solution search, this is the numeric value
-        of the objective function. If the objective function is not fully valued
-        or if the problem is a feasiblity problem without an objective function,
-        this is :obj:`None`.
-
-        In the case of an uncertain objective, this is the worst-case (expected)
-        objective value.
-
-        :raises picos.uncertain.IntractableWorstCase:
-            When computing the worst-case (expected) value of an uncertain
-            objective is not supported.
-
-        .. note::
-
-            The Python special functions :class:`int`, :class:`float`,
-            :class:`complex` and :func:`round` as well as the special method
-            ``__index__`` make use of this value when applied to a
-            :class:`Problem`.
-        """
-        return self._objective.value
-
     @property
     def options(self):
         """Solution search parameters as an :class:`~picos.Options` object."""
@@ -682,8 +690,21 @@ class Problem():
             template, data = parameterized_string(
                 [mtb.long_string for mtb in group])
         except ValueError:
-            return group[0].long_string \
-                + ", " + ", ".join([v.name for v in group[1:]])
+            # HACK: Use the plural of the type string (e.g. "real variables").
+            type_string = group[0]._get_type_string_base().lower()
+            base_string = group[0].long_string.replace(
+                type_string, type_string + "s")
+
+            # HACK: Move any bound string to the end.
+            match = re.match(r"([^(]*)( \([^)]*\))", base_string)
+            if match:
+                base_string = match[1]
+                bound_string = match[2]
+            else:
+                bound_string = ""
+
+            return base_string \
+                + ", " + ", ".join([v.name for v in group[1:]]) + bound_string
         else:
             return glyphs.forall(template, data)
 
@@ -704,11 +725,17 @@ class Problem():
             return glyphs.forall(template, data)
 
     def __repr__(self):
-        return glyphs.repr1(self.type)
+        if self._name:
+            return glyphs.repr2(self.type, self._name)
+        else:
+            return glyphs.repr1(self.type)
 
     def __str__(self):
-        # Print problem type.
-        string = "{}\n".format(self.type)
+        # Print problem name (if available) and type.
+        if self._name:
+            string = "{} ({})\n".format(self._name, self.type)
+        else:
+            string = "{}\n".format(self.type)
 
         # Print objective.
         string += "  {}\n".format(self._objective)
@@ -743,21 +770,6 @@ class Problem():
 
         return string.rstrip("\n")
 
-    def __index__(self):
-        return self._objective.__index__()
-
-    def __int__(self):
-        return self._objective.__int__()
-
-    def __float__(self):
-        return self._objective.__float__()
-
-    def __complex__(self):
-        return self._objective.__complex__()
-
-    def __round__(self, ndigits=None):
-        return self._objective.__round__(ndigits)
-
     def __iadd__(self, constraints):
         if isinstance(constraints, tuple):
             self.require(*constraints)
@@ -1303,7 +1315,7 @@ class Problem():
         Linear Program
           minimize x + y[0] + y[1] + y[2] + y[3] + z[alice] + z[bob] + z[carol]
           over
-            1×1 real variable x, y[0], y[1], y[2], y[3], z[alice], z[bob],
+            1×1 real variables x, y[0], y[1], y[2], y[3], z[alice], z[bob],
               z[carol]
         >>> # Retrieve the variables from the problem.
         >>> P.get_variable("x")
@@ -1355,7 +1367,7 @@ class Problem():
         """Retrieve values of variables referenced by the problem.
 
         This method works the same :meth:`get_variable` but it returns the
-        variable's :attr:`values <.expression.Expression.value>` instead of the
+        variable's :attr:`values <.valuable.Valuable.value>` instead of the
         variable objects.
 
         :raises ~picos.expressions.NotValued:
@@ -1700,9 +1712,9 @@ class Problem():
         access the solution as follows:
 
             - The problem's :attr:`value` denotes the objective function value.
-            - The variables' :attr:`~.expression.Expression.value` is set
-              according to the primal solution. You can in fact query the value
-              of any expression involving valued variables like this.
+            - The variables' :attr:`~.valuable.Valuable.value` is set according
+              to the primal solution. You can in fact query the value of any
+              expression involving valued variables like this.
             - The constraints' :attr:`~.constraint.Constraint.dual` is set
               according to the dual solution.
             - The value of any parameter involved in the problem may have
@@ -1814,21 +1826,25 @@ class Problem():
 
             # Validate the primal solution.
             if options.primals:
+                vars_ = self._variables.values()
                 if solution.primalStatus != SS_OPTIMAL:
                     raise SolutionFailure(3, "Primal solution state claimed {} "
                         "but optimality is required (primals=True)."
                         .format(solution.primalStatus))
-                elif None in solution.primals.values():
+                elif None in solution.primals.values() \
+                or any(var not in solution.primals for var in vars_):
                     raise SolutionFailure(3, "The primal solution is incomplete"
                         " but full primals are required (primals=True).")
 
             # Validate the dual solution.
             if options.duals:
+                cons = self._constraints.values()
                 if solution.dualStatus != SS_OPTIMAL:
                     raise SolutionFailure(4, "Dual solution state claimed {} "
                         "but optimality is required (duals=True).".format(
                         solution.dualStatus))
-                elif None in solution.duals.values():
+                elif None in solution.duals.values() \
+                or any(con not in solution.duals for con in cons):
                     raise SolutionFailure(4, "The dual solution is incomplete "
                         "but full duals are required (duals=True).")
 
@@ -1960,6 +1976,16 @@ class Problem():
 
         return (not largest_violation, largest_violation)
 
+    # --------------------------------------------------------------------------
+    # Abstract method implementations for the Valuable base class.
+    # --------------------------------------------------------------------------
+
+    def _get_valuable_string(self):
+        return "problem with {}".format(self._objective._get_valuable_string())
+
+    def _get_value(self):
+        return self._objective._get_value()
+
     # --------------------------------------------------------------------------
     # Legacy methods and properties.
     # --------------------------------------------------------------------------
@@ -2159,7 +2185,7 @@ class Problem():
 
     @deprecated("2.0", useInstead="variables")
     def set_var_value(self, name, value):
-        """Set the :attr:`value <.expression.Expression.value>` of a variable.
+        """Set the :attr:`~.valuable.Valuable.value` of a variable.
 
         For a :class:`Problem` ``P``, this is the same as
         ``P.variables[name] = value``.
