@@ -103,7 +103,7 @@ def find_assignment(subject_to=[], **options):
 
     :returns:
         Nothing. Check the concerned variables' :attr:`values
-        <.expression.Expression.value>`.
+        <.valuable.Valuable.value>`.
 
     :raises ~picos.SolutionFailure:
         See :meth:`~.problem.Problem.solve`.
