@@ -135,8 +135,7 @@ class Solution:
     the user:
 
     >>> import picos
-    >>> P = picos.Problem()
-    >>> x = P.add_variable("x")
+    >>> x = picos.RealVariable("x")
     >>> s = picos.Solution({x: 1}); s
     <detached primal solution from user>
     >>> s.apply()
@@ -146,12 +145,14 @@ class Solution:
     If the solution was created by a solver (or attached to a problem via
     :func:`attach_to`), more information is available:
 
-    >>> C1 = P.add_constraint(x >= 2)
-    >>> s = P.minimize(x, solver = "cvxopt", duals = False); s
+    >>> P = picos.Problem()
+    >>> P.minimize = x
+    >>> P += x >= 2
+    >>> s = P.solve(solver = "cvxopt", duals = False); s
     <feasible primal solution (claimed optimal) from cvxopt>
     >>> "{:.2f} ms".format(1000.0 * s.searchTime) #doctest: +SKIP
     '0.83 ms'
-    >>> C2 = P.add_constraint(x >= 3); s
+    >>> P += x >= 3; s
     <infeasible primal solution (was feasible and claimed optimal) from cvxopt>
     """
 
