@@ -44,6 +44,7 @@ from .solver_glpk   import GLPKSolver  # noqa
 from .solver_gurobi import GurobiSolver  # noqa
 from .solver_mosek  import MOSEKSolver  # noqa
 from .solver_mskfsn import MOSEKFusionSolver  # noqa
+from .solver_osqp   import OSQPSolver  # noqa
 from .solver_scip   import SCIPSolver  # noqa
 from .solver_smcp   import SMCPSolver  # noqa
 
@@ -57,6 +58,7 @@ _solvers = {solver.names()[0]: solver for solver in (
     GurobiSolver,
     MOSEKSolver,
     MOSEKFusionSolver,
+    OSQPSolver,
     SCIPSolver,
     SMCPSolver
 )}
