@@ -1,35 +1,32 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2017-2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Backend for solver interface implementations."""
 
-from __future__ import print_function
-
+import importlib.util
 import os
 import sys
 import time
+from abc import ABC, abstractmethod
 from contextlib import contextmanager
 
+from .. import settings
 from ..apidoc import api_end, api_start
-from ..compat import ABC, abstractmethod
 from ..formatting import solver_box
 from ..modeling.solution import Solution
 
@@ -114,23 +111,47 @@ class OptionValueError(OptionError, ValueError):
 class Solver(ABC):
     """Base class for an interface to an optimization solver."""
 
+    # --------------------------------------------------------------------------
+    # Static methods.
+    # --------------------------------------------------------------------------
+
+    @staticmethod
+    def check_import(importName):
+        """Asserts that a module is available without actually importing it.
+
+        :raises ModuleNotFoundError:
+            If the module could not be found.
+        """
+        if importlib.util.find_spec(importName) is None:
+            raise ModuleNotFoundError(
+                "Python module '{}' not found.".format(importName))
+
     # --------------------------------------------------------------------------
     # Abstract class methods.
     # --------------------------------------------------------------------------
 
     @classmethod
     @abstractmethod
-    def supports(cls, footprint):
+    def supports(cls, footprint, explain=False):
         """Whether a type of problem, given by footprint, is supported.
 
         The default implementation ensures that all reformulations required by
         user's choice have been performed before the problem is handed to the
         solver. Solver implementations are thus required to incorporate it.
+
+        :param bool explain:
+            If :obj:`True`, then this returns a :class:`tuple` where the first
+            element is this method's regular return value and where the second
+            element is a string naming one reason why the footprint is not
+            supported (:obj:`None` if it is).
         """
         if footprint.options.dualize:
-            return False
+            if explain:
+                return False, "Variants of the primal problem (dualize=True)."
+            else:
+                return False
 
-        return True
+        return (True, None) if explain else True
 
     @classmethod
     @abstractmethod
@@ -158,6 +179,17 @@ class Solver(ABC):
         """Returns a triple ``(name, display_name, long_display_name)``."""
         pass
 
+    @classmethod
+    @abstractmethod
+    def is_free(cls):
+        """Report whether the solver is free software.
+
+        This allows users to prevent PICOS from using non-free solvers at all,
+        including for internal use, via the :data:`~.settings.NONFREE_SOLVERS`
+        setting.
+        """
+        pass
+
     # --------------------------------------------------------------------------
     # __init__ and instance properties.
     # --------------------------------------------------------------------------
@@ -182,8 +214,8 @@ class Solver(ABC):
 
         # The external (PICOS) problem represenation.
         # HACK: Quick and dirty conversion to accept reformulations as input.
-        from ..reforms import Reformulation
         from ..modeling import Problem
+        from ..reforms import Reformulation
         if isinstance(problem, Reformulation):
             problem.successor = self
             self.predecessor  = problem
@@ -317,14 +349,32 @@ class Solver(ABC):
     @classmethod
     def available(cls, verbose=False):
         """Whether the solver is properly installed on the system."""
+        name, display_name, _ = cls.names()
+
+        if name in settings.SOLVER_BLACKLIST:
+            if verbose:
+                print("The solver {} is blacklisted.".format(display_name))
+            return False
+
+        if settings.SOLVER_WHITELIST and name not in settings.SOLVER_WHITELIST:
+            if verbose:
+                print("The solver {} is not whitelisted.".format(display_name))
+            return False
+
+        if not settings.NONFREE_SOLVERS and not cls.is_free():
+            if verbose:
+                print("The solver {} is non-free.".format(display_name))
+            return False
+
         try:
             cls.test_availability()
-            return True
-        except Exception as e:
+        except Exception as error:
             if verbose:
-                print(e)
+                print(error)
             return False
 
+        return True
+
     @classmethod
     def predict(cls, footprint):
         """Return the solver class.
@@ -421,13 +471,33 @@ class Solver(ABC):
         for option in options:
             self._handle_unsupported_option(option)
 
-    def _handle_bad_option_value(self, option, reason=None):
-        """Inform the user that they set an option to an invalid value."""
-        assert option in self.ext.options, \
-            "The option '{}' does not exist.".format(option)
-        raise OptionValueError("Invalid value '{}' for option '{}'{}"
-            .format(self.ext.options[option], option,
-            ": {}".format(reason) if reason else "."))
+    def _handle_bad_solver_specific_option(self, key, value, error):
+        picos_option = "{}_params".format(self.name)
+        assert picos_option in self.ext.options, \
+            "The PICOS option '{}' does not exist.".format(picos_option)
+
+        raise OptionValueError(
+            "Either the {} option '{}' set via '{}' does not exist or the given"
+            " value '{}' is not valid for that option.".format(
+            self.display_name, key, picos_option, value)) from error
+
+    def _handle_bad_solver_specific_option_key(self, key, error=None):
+        picos_option = "{}_params".format(self.name)
+        assert picos_option in self.ext.options, \
+            "The PICOS option '{}' does not exist.".format(picos_option)
+
+        raise OptionValueError(
+            "The {} option '{}' set via '{}' does not exist.".format(
+            self.display_name, key, picos_option)) from error
+
+    def _handle_bad_solver_specific_option_value(self, key, value, error=None):
+        picos_option = "{}_params".format(self.name)
+        assert picos_option in self.ext.options, \
+            "The PICOS option '{}' does not exist.".format(picos_option)
+
+        raise OptionValueError(
+            "Invalid value '{}' for {} option '{}' set via '{}'.".format(
+            value, self.display_name, key, picos_option)) from error
 
     def _handle_continuous_nonconvex_error(self, error):
         """Raise a descriptive :exc:`ArithmeticError`."""
@@ -437,8 +507,7 @@ class Solver(ABC):
             "quadratic form is numerically on the verge of being semidefinite, "
             "with PICOS' and {0}'s judgement differing. You could try a slight "
             "perturbation of your data such that all quadratic forms become "
-            "definite. The error produced by {0} reads: {1}".format(
-            self.display_name, error))
+            "definite.".format(self.display_name)) from error
 
     def _load_problem(self):
         """(Re-)import or update the solver's problem state for solving."""
@@ -569,8 +638,8 @@ class Solver(ABC):
         """Reset the timer of the ``_stopwatch`` context manager."""
         self.timer = None
 
-    def _make_solution(self, primals, duals, primalStatus, dualStatus,
-            problemStatus, info={}, vectorizedPrimals=True):
+    def _make_solution(self, value, primals, duals, primalStatus, dualStatus,
+            problemStatus, info=None, vectorizedPrimals=True):
         """Create a solution problem from within :meth:`_solve`.
 
         Note that the default value for ``vectorizedPrimals`` is :obj:`True`,
@@ -585,9 +654,18 @@ class Solver(ABC):
         assert self.timer is not None, \
             "Solvers must measure search time via _stopwatch."
 
-        return Solution(primals, duals, self.ext, get_solver_name(self),
-            primalStatus, dualStatus, problemStatus, self.timer, info,
-            vectorizedPrimals)
+        return Solution(
+            primals=primals,
+            duals=duals,
+            problem=self.ext,
+            solver=get_solver_name(self),
+            primalStatus=primalStatus,
+            dualStatus=dualStatus,
+            problemStatus=problemStatus,
+            searchTime=self.timer,
+            info=info,
+            vectorizedPrimals=vectorizedPrimals,
+            reportedValue=value)
 
     def execute(self):
         """Solve the problem and return the solution.
@@ -616,12 +694,22 @@ class Solver(ABC):
                 subsolver, self._verbose()):
             yield
 
+    @property
+    def _license_warnings(self):
+        """Whether license related warnings may ignore verbosity."""
+        return settings.LICENSE_WARNINGS and self.ext.options.license_warnings
+
     @contextmanager
     def _enforced_verbosity(self, noStdOutAt=0, noStdErrAt=-1):
         """Enfoce the user-specified verbosity within the context.
 
         :param int noStdOutAt: Don't print to stdout at or below this verbosity.
         :param int noStdErrAt: Don't print to stderr at or below this verbosity.
+
+        .. warning::
+
+            This context manager monkey-patches the :mod:`sys` module and is not
+            thread safe.
         """
         verbosity = self.ext.verbosity()
 
@@ -633,16 +721,17 @@ class Solver(ABC):
             sys.stdout = devNull
 
         if verbosity <= noStdErrAt:
-            oldErr = sys.sdterr
+            oldErr = sys.stderr
             sys.stderr = devNull
 
-        yield
+        try:
+            yield
+        finally:
+            if verbosity <= noStdOutAt:
+                sys.stdout = oldOut
 
-        if verbosity <= noStdOutAt:
-            sys.stdout = oldOut
-
-        if verbosity <= noStdErrAt:
-            sys.stderr = oldErr
+            if verbosity <= noStdErrAt:
+                sys.stderr = oldErr
 
 
 # --------------------------------------
