@@ -103,7 +103,6 @@ class CPLEXSolver(Solver):
         constraints since indices can change if the CPLEX instance is modified.
     """
 
-    # TODO: Allow nonconvex quadratic constraints in the integer case?
     # NOTE: When making changes, also see the section in _solve that tells CPLEX
     #       the problem type.
     SUPPORTED = Specification(
@@ -117,6 +116,10 @@ class CPLEXSolver(Solver):
             RSOCConstraint,
             ConvexQuadraticConstraint])
 
+    NONCONVEX_QP = Specification(
+        objectives=[QuadraticExpression],
+        constraints=[DummyConstraint, AffineConstraint])
+
     @classmethod
     def supports(cls, footprint, explain=False):
         """Implement :meth:`~.solver.Solver.supports`."""
@@ -124,10 +127,14 @@ class CPLEXSolver(Solver):
         if not result or (explain and not result[0]):
             return result
 
-        if footprint.nonconvex_quadratic_objective and footprint.continuous:
+        # Support QPs and MIQPs with a nonconvex objective.
+        # NOTE: SUPPORTED fully excludes nonconvex quadratic constraints. This
+        #       further excludes QCQPs and MIQCQPs with a nonconvex objective.
+        # TODO: See which of the excluded cases can be supported as well.
+        if footprint.nonconvex_quadratic_objective \
+        and footprint not in cls.NONCONVEX_QP:
             if explain:
-                return (False,
-                    "Continuous problems with nonconvex quadratic objectives.")
+                return (False, "(MI)QCQPs with nonconvex objective.")
             else:
                 return False
 
@@ -724,6 +731,9 @@ class CPLEXSolver(Solver):
 
         continuous = self.ext.is_continuous()
 
+        # TODO: Allow querying self.ext.objective directly.
+        nonconvex_quad_obj = self.ext.footprint.nonconvex_quadratic_objective
+
         # verbosity
         verbosity = self.verbosity()
         if verbosity <= 0:
@@ -846,6 +856,15 @@ class CPLEXSolver(Solver):
                     cplex.SparsePair(ind=names, val=values),
                     self.int.MIP_starts.effort_level.repair)
 
+        # Set the optimality target now so that cplex_params may overwrite it.
+        # This allows solving QPs and MIQPs with a nonconvex objective.
+        if nonconvex_quad_obj:
+            p.optimalitytarget.set(3)
+
+        # Load a virtual machine config.
+        if self.ext.options.cplex_vmconfig:
+            self.int.copy_vmconfig(self.ext.options.cplex_vmconfig)
+
         # Handle CPLEX-specific options.
         for key, value in o.cplex_params.items():
             try:
@@ -913,6 +932,15 @@ class CPLEXSolver(Solver):
                 # Assume quadratic constraint types.
                 cplexType = cplexTypes.QCP if continuous else cplexTypes.MIQCP
 
+        # Silence a warning explaining that optimality target 3 changes the
+        # problem type from QP to MIQP by doing so manually.
+        if nonconvex_quad_obj:
+            # Enforce consistency with CPLEXSolver.supports.
+            assert cplexType in (cplexTypes.QP, cplexTypes.MIQP)
+
+            if p.optimalitytarget.get() == 3:  # User might have changed it.
+                cplexType = cplexTypes.MIQP
+
         if cplexType is not None:
             self.int.set_problem_type(cplexType)
 
