@@ -29,8 +29,8 @@ from ..expressions import CONTINUOUS_VARTYPES, AffineExpression, LogSumExp
 from ..modeling.footprint import Specification
 from ..modeling.solution import (PS_FEASIBLE, PS_INFEASIBLE, PS_UNBOUNDED,
                                  PS_UNKNOWN, SS_INFEASIBLE, SS_OPTIMAL,
-                                 SS_UNKNOWN)
-from .solver import Solver, SolverError
+                                 SS_UNKNOWN, Solution)
+from .solver import Solver
 
 _API_START = api_start(globals())
 # -------------------------------
@@ -474,9 +474,14 @@ class CVXOPTSolver(Solver):
             JP = list(set(p["A"].I))
             IP = range(len(JP))
             VP = [1] * len(JP)
+
+            # Skip solution on an infeasible constraint.
             if any([b for (i, b) in enumerate(p["b"]) if i not in JP]):
-                raise SolverError(
-                    "Infeasible constraint of the form 0 = a with a != 0.")
+                return Solution(
+                    primals=None, duals=None, problem=self.ext, solver="PICOS",
+                    primalStatus=SS_INFEASIBLE, dualStatus=SS_UNKNOWN,
+                    problemStatus=PS_INFEASIBLE, vectorizedPrimals=True)
+
             P = cvxopt.spmatrix(VP, IP, JP, (len(IP), p["A"].size[0]))
             A = P * p["A"]
             b = P * p["b"]
