@@ -516,6 +516,7 @@ class MOSEKFusionSolver(Solver):
         #       is necessary as the Fusion API has no call to reset options.
         # TODO: As soon as the Fusion API offers option reset, use it instead.
         self.int.getTask().setdefaults()
+        self.int.optserverHost("")
 
         # verbosity
         self.int.setSolverParam("log", max(0, self.verbosity()))
@@ -636,6 +637,11 @@ class MOSEKFusionSolver(Solver):
             except ValueError as error:
                 self._handle_bad_solver_specific_option_value(key, value, error)
 
+        # Handle 'mosek_server' option.
+        # FIXME: This produces unsolicited console output with MOSEK 9.2.
+        if self.ext.options.mosek_server:
+            self.int.optserverHost(self.ext.options.mosek_server)
+
         # Handle unsupported options.
         self._handle_unsupported_option("treememory")
 
@@ -671,15 +677,13 @@ class MOSEKFusionSolver(Solver):
                     duals[picosCon] = None
                     continue
 
-                # Devectorize the dual. Note the change from row-major to
-                # column-major order.
-                # size = picosCon.size
-                # picosDual = cvxopt.matrix(mosekDual, (size[1], size[0])).T
-                picosDual = cvxopt.matrix(mosekDual)
+                # Devectorize the dual.
+                # NOTE: Change from row-major to column-major order.
+                size = picosCon.size
+                picosDual = cvxopt.matrix(mosekDual, (size[1], size[0])).T
 
                 # Adjust the dual based on constraint type.
-                if isinstance(picosCon, AffineConstraint) \
-                or isinstance(picosCon, LMIConstraint):
+                if isinstance(picosCon, (AffineConstraint, LMIConstraint)):
                     if not picosCon.is_increasing():
                         picosDual = -picosDual
                 elif isinstance(picosCon, SOCConstraint):
