@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2018-2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implementation of :class:`SCIPSolver`."""
@@ -24,16 +21,15 @@
 import cvxopt
 
 from ..apidoc import api_end, api_start
-from ..compat import assert_import_exists
 from ..constraints import (AffineConstraint, ConvexQuadraticConstraint,
-                           NonconvexQuadraticConstraint, RSOCConstraint,
-                           SOCConstraint)
+                           DummyConstraint, NonconvexQuadraticConstraint,
+                           RSOCConstraint, SOCConstraint)
 from ..expressions import AffineExpression, BinaryVariable, IntegerVariable
 from ..modeling.footprint import Specification
 from ..modeling.solution import (PS_FEASIBLE, PS_INFEASIBLE, PS_UNBOUNDED,
                                  PS_UNKNOWN, SS_INFEASIBLE, SS_OPTIMAL,
                                  SS_PREMATURE, SS_UNKNOWN)
-from .solver import Solver
+from .solver import OptionValueError, Solver
 
 _API_START = api_start(globals())
 # -------------------------------
@@ -42,12 +38,11 @@ _API_START = api_start(globals())
 class SCIPSolver(Solver):
     """Interface to the SCIP solver via the PySCIPOpt Python interface."""
 
-    SUPPORTED = Specification.compile(
-        directions=Specification.ALL,
+    SUPPORTED = Specification(
         objectives=[
             AffineExpression],
-        variables=Specification.ALL,
         constraints=[
+            DummyConstraint,
             AffineConstraint,
             SOCConstraint,
             RSOCConstraint,
@@ -55,21 +50,32 @@ class SCIPSolver(Solver):
             NonconvexQuadraticConstraint])
 
     @classmethod
-    def supports(cls, footprint):
+    def supports(cls, footprint, explain=False):
         """Implement :meth:`~.solver.Solver.supports`."""
-        if not Solver.supports(footprint):
-            return False
+        result = Solver.supports(footprint, explain)
+        if not result or (explain and not result[0]):
+            return result
 
-        # HACK: SCIP does not produce dual values (except for LPs, where they
-        #       are known to be sometimes incorrect). For now pretend SCIP does
-        #       not support continuous problems at all (unless there are
-        #       nonconvex quadratics where no other solver works).
-        # TODO: Support all continuous problems when ``duals=False``.
+        # HACK: SCIP has trouble with continuous problems, in particular it only
+        #       produces dual values for LPs (wehich are sometimes incorrect).
+        #       Nonconvex quadratics are supported as no other solver can deal
+        #       with them at this point.
         if footprint.continuous \
         and not ("con", NonconvexQuadraticConstraint) in footprint:
-            return False
+            if explain:
+                return (False,
+                    "Continuous problems, except for nonconvex quadratic ones "
+                    "(PICOS' choice).")
+            else:
+                return False
 
-        return footprint << cls.SUPPORTED
+        if footprint not in cls.SUPPORTED:
+            if explain:
+                return False, cls.SUPPORTED.mismatch_reason(footprint)
+            else:
+                return False
+
+        return (True, None) if explain else True
 
     @classmethod
     def default_penalty(cls):
@@ -79,13 +85,18 @@ class SCIPSolver(Solver):
     @classmethod
     def test_availability(cls):
         """Implement :meth:`~.solver.Solver.test_availability`."""
-        assert_import_exists("pyscipopt")
+        cls.check_import("pyscipopt")
 
     @classmethod
     def names(cls):
         """Implement :meth:`~.solver.Solver.names`."""
         return "scip", "SCIP", "SCIP Optimization Suite"
 
+    @classmethod
+    def is_free(cls):
+        """Implement :meth:`~.solver.Solver.is_free`."""
+        return False
+
     def __init__(self, problem):
         """Initialize a SCIP solver interface.
 
@@ -220,7 +231,7 @@ class SCIPSolver(Solver):
 
         # Convert quadratic part.
         for (pVar1, pVar2), pCoefficients \
-        in picosExpression.quadratic_forms.items():
+        in picosExpression._sparse_quads.items():
             for sparseIndex in range(len(pCoefficients)):
                 localVar1Index   = pCoefficients.I[sparseIndex]
                 localVar2Index   = pCoefficients.J[sparseIndex]
@@ -301,7 +312,9 @@ class SCIPSolver(Solver):
         elif isinstance(picosConstraint, RSOCConstraint):
             scipCons = self._import_rscone_constraint(picosConstraint)
         else:
-            assert False, "Constraint type belongs to unsupported problem type."
+            assert isinstance(picosConstraint, DummyConstraint), \
+                "Unexpected constraint type: {}".format(
+                picosConstraint.__class__.__name__)
 
         # Map PICOS constraints to lists of SCIP constraints.
         self._scipCons[picosConstraint] = scipCons
@@ -319,14 +332,6 @@ class SCIPSolver(Solver):
         # Import objective function.
         scipObjective = self._scalar_affinexp_pic2scip(picosObjective)
 
-        # HACK: Remove a constant term from the objective as this is not allowed
-        #       by SCIP under Python 2. (Not sure if the term is stored or
-        #       dropped under Python 3, but no exception is thrown there.)
-        from pyscipopt.scip import Term
-        constantTerm = Term()
-        if constantTerm in scipObjective.terms:
-            scipObjective.terms.pop(constantTerm)
-
         self.int.setObjective(scipObjective, scipSense)
 
     def _import_problem(self):
@@ -423,38 +428,43 @@ class SCIPSolver(Solver):
                 "limits/solutions", float(self.ext.options.max_fsb_nodes))
 
         # Handle SCIP-specific options.
-        for option, value in self.ext.options.scip_params.items():
+        for key, value in self.ext.options.scip_params.items():
             try:
                 if isinstance(value, bool):
-                    self.int.setBoolParam(option, value)
+                    self.int.setBoolParam(key, value)
                 elif isinstance(value, str):
                     if len(value) == 1:
                         try:
-                            self.int.setCharParam(option, ord(value))
+                            self.int.setCharParam(key, ord(value))
                         except LookupError:
-                            self.int.setStringParam(option, value)
+                            self.int.setStringParam(key, value)
                     else:
-                        self.int.setStringParam(option, value)
+                        self.int.setStringParam(key, value)
                 elif isinstance(value, float):
-                    self.int.setRealParam(option, value)
+                    self.int.setRealParam(key, value)
                 elif isinstance(value, int):
                     try:
-                        self.int.setIntParam(option, value)
+                        self.int.setIntParam(key, value)
                     except LookupError:
                         try:
-                            self.int.setLongintParam(option, value)
+                            self.int.setLongintParam(key, value)
                         except LookupError:
-                            self.int.setRealParam(option, float(value))
-            except KeyError:
-                self._handle_bad_option_value("scip_params",
-                    "SCIP option '{}' does not exist.".format(option))
-            except ValueError:
-                self._handle_bad_option_value("scip_params",
-                    "Invalid value '{}' for SCIP option '{}'."
-                    .format(value, option))
-            except LookupError:
-                self._handle_bad_option_value("scip_params",
-                    "Failed to guess type of SCIP option '{}'.".format(option))
+                            self.int.setRealParam(key, float(value))
+                else:
+                    raise TypeError(
+                        "The value '{}' has an uncommon type.".format(value))
+            except KeyError as error:
+                self._handle_bad_solver_specific_option_key(key, error)
+            except ValueError as error:
+                self._handle_bad_solver_specific_option_value(key, value, error)
+            except (TypeError, LookupError) as error:
+                picos_option = "{}_params".format(self.name)
+                assert picos_option in self.ext.options, \
+                    "The PICOS option '{}' does not exist.".format(picos_option)
+
+                raise OptionValueError(
+                    "Failed to guess type of {} option '{}' set via '{}'."
+                    .format(self.display_name, key, picos_option)) from error
 
         # Handle unsupported options.
         self._handle_unsupported_options(
@@ -494,6 +504,11 @@ class SCIPSolver(Solver):
         duals = {}
         if self.ext.options.duals is not False and isLP:
             for picosConstraint in self.ext.constraints.values():
+                if isinstance(picosConstraint, DummyConstraint):
+                    duals[picosConstraint] = cvxopt.spmatrix(
+                        [], [], [], picosConstraint.size)
+                    continue
+
                 assert isinstance(picosConstraint, AffineConstraint)
 
                 # Retrieve dual value for constraint.
@@ -512,16 +527,10 @@ class SCIPSolver(Solver):
 
                 duals[picosConstraint] = picosDual
 
-        # # Retrieve objective value.
-        # objectiveValue = self.int.getObjVal()
-        #
-        # # HACK: Add back the constant objective term; see above.
-        # picosObjective = self.ext.no.function
-        # picosObjectiveAffinePart = picosObjective.aff \
-        #     if isinstance(picosObjective, QuadExp) else picosObjective
-        # if picosObjectiveAffinePart.constant:
-        #     objectiveValue += picosObjectiveAffinePart.constant[0]
+        # Retrieve objective value.
+        value = self.int.getObjVal()
 
+        # Retrieve solution status.
         status = self.int.getStatus()
         if   status == "optimal":
             primalStatus   = SS_OPTIMAL
@@ -545,7 +554,7 @@ class SCIPSolver(Solver):
             problemStatus  = PS_UNKNOWN
 
         return self._make_solution(
-            primals, duals, primalStatus, dualStatus, problemStatus)
+            value, primals, duals, primalStatus, dualStatus, problemStatus)
 
 
 # --------------------------------------
