@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2017 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implementation of :class:`SMCPSolver`."""
@@ -27,8 +24,7 @@
 # ------------------------------------------------------------------------------
 
 from ..apidoc import api_end, api_start
-from ..compat import assert_import_exists
-from ..constraints import AffineConstraint, LMIConstraint
+from ..constraints import AffineConstraint, DummyConstraint, LMIConstraint
 from ..expressions import CONTINUOUS_VARTYPES, AffineExpression
 from ..modeling.footprint import Specification
 from .solver import Solver
@@ -45,12 +41,12 @@ class SMCPSolver(CVXOPTSolver):
     :class:`CVXOPTSolver <picos.solvers.solver_cvxopt.CVXOPTSolver>` base class.
     """
 
-    SUPPORTED = Specification.compile(
-        directions=Specification.ALL,
+    SUPPORTED = Specification(
         objectives=[
             AffineExpression],
         variables=CONTINUOUS_VARTYPES,
         constraints=[
+            DummyConstraint,
             AffineConstraint,
             # TODO: See below.
             # SOCConstraint,
@@ -58,18 +54,28 @@ class SMCPSolver(CVXOPTSolver):
             LMIConstraint])
 
     @classmethod
-    def supports(cls, footprint):
+    def supports(cls, footprint, explain=False):
         """Implement :meth:`~.solver.Solver.supports`."""
-        if not Solver.supports(footprint):
-            return False
+        result = Solver.supports(footprint, explain)
+        if not result or (explain and not result[0]):
+            return result
 
         # TODO: SMCP formally supports problems that are not SDPs, but it is
         #       known to perform poorly for them, sometimes even returning
         #       incorrect solutions. For now, only support SDPs.
         if not ("con", LMIConstraint) in footprint:
-            return False
+            if explain:
+                return False, "Problems other than SDPs (PICOS' choice)."
+            else:
+                return False
 
-        return footprint << cls.SUPPORTED
+        if footprint not in cls.SUPPORTED:
+            if explain:
+                return False, cls.SUPPORTED.mismatch_reason(footprint)
+            else:
+                return False
+
+        return (True, None) if explain else True
 
     @classmethod
     def default_penalty(cls):
@@ -79,13 +85,18 @@ class SMCPSolver(CVXOPTSolver):
     @classmethod
     def test_availability(cls):
         """Implement :meth:`~.solver.Solver.test_availability`."""
-        assert_import_exists("smcp")
+        cls.check_import("smcp")
 
     @classmethod
     def names(cls):
         """Implement :meth:`~.solver.Solver.names`."""
         return "smcp", "SMCP", "Sparse Matrix Cone Program Solver"
 
+    @classmethod
+    def is_free(cls):
+        """Implement :meth:`~.solver.Solver.is_free`."""
+        return True
+
     @property
     def is_smcp(self):
         """Whether to implement SMCP instead of CVXOPT."""
